#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <libgen.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <stdint.h>
#include <stdbool.h>
#include <fcntl.h>
//----------------------------------------------------------------------
static void usage (void);
char buffer[BUFSIZ];
//----------------------------------------------------------------------
int
main (int argc, char *argv[], char *envp[])
{
  bool               option_l = 0;
  int                opt;
//extern char       *optarg;              // not used.
  extern int         optind;
  extern int         optopt;
  //
  int                status;
  int                sfdn;
  int                sfdn2;
  struct sockaddr_un sa_unix;
  socklen_t          sa_unix_size;
  ssize_t            read_size;
  ssize_t            sent_size;
  ssize_t            recv_size;
  char              *path = NULL;
  bool               can_rx = 1;
  bool               can_tx = 1;
  //
  // Check for options.
  //
  while ((opt = getopt (argc, argv, ":l")) != -1)
    {
      switch (opt)
	{
	case 'l':
	  option_l = 1;
	  break;
	case '?':
	  fprintf (stderr, "Unknown option -%c.\n", optopt);
	  usage ();
	  return (1);
	  break;
	case ':':
	  fprintf (stderr, "Missing argument for option -%c\n", optopt);
	  usage ();
	  return (1);
	  break;
	default:
	  fprintf (stderr, "Getopt problem: unknown option %c\n", opt);
	  usage ();
	  return (1);
	}
    }
  //
  // Arguments.
  //
  if (optind == (argc - 1))
    {
      //
      // There is exactly one argument: the unix domain socket path.
      //
      path = argv[argc-1];
      //
      // The path cannot be longer than the `sun_path[]' member.
      //
      if (strlen (path) >= sizeof (sa_unix.sun_path))
        {
          printf ("The socket path is too long for the library! "
                  "It can be max %u bytes.\n",
                  (unsigned int) (sizeof (sa_unix.sun_path) - 1));
        }
    }
  else
    {
      //
      // Arguments wrong!
      //
      usage ();
      return (2);
    }
  //
  // Set the local or the remote address.
  //
  sa_unix.sun_family = AF_UNIX;
  strcpy (sa_unix.sun_path, path);
  sa_unix_size = sizeof (struct sockaddr_un);
  //
  // Open the socket.
  //
  sfdn = socket (AF_UNIX, SOCK_STREAM, 0);
  if (sfdn < 0)
    {
      perror (NULL);
      return (3);
    }
  //
  // Set it listening or connect.
  //
  if (option_l)
    {
      //
      // Bind the local `sa' location.
      //
      status = bind (sfdn, (struct sockaddr *) &sa_unix,
                     sizeof (sa_unix));
      if (status < 0)
        {
          perror (NULL);
          close (sfdn);
          return (4);
        }
      //
      // Listen.
      //
      status = listen (sfdn, 1);
      if (status < 0)
        {
          perror (NULL);
          close (sfdn);
          return (5);
        }
      //
      // Accept.
      //
      sfdn2 = accept (sfdn, NULL, NULL);
      if (sfdn2 < 0)
        {
          perror (NULL);
          close (sfdn);
          return (6);
        }
      //
      // Close listening socket.
      //
      close (sfdn);
      //
      // Variable `sfdn' will be the new socket.
      //
      sfdn = sfdn2;
    }
  else
    {
      //
      // Connect the destination.
      //
      status = connect (sfdn, (struct sockaddr *) &sa_unix,
                        sizeof (sa_unix));
      if (status < 0)
        {
          perror (NULL);
          close (sfdn);
          return (7);
        }
    }
  //
  // Define the standard input non blocking.
  //
  status = fcntl (STDIN_FILENO, F_SETFL, O_NONBLOCK);
  if (status < 0)
    {
      perror (NULL);
      return (8);
    }
  //
  // Define the socket non blocking.
  //
  status = fcntl (sfdn, F_SETFL, O_NONBLOCK);
  if (status < 0)
    {
      perror (NULL);
      return (9);
    }
  //
  // Will read from the remote and show to the screen.
  //
  while (can_rx || can_tx)
    {
      if (can_rx)
        {
          recv_size = recv (sfdn, &buffer, (size_t) BUFSIZ-1, 0);
//        recv_size = read (sfdn, &buffer, (size_t) BUFSIZ-1);
          if (recv_size < 0)
            {
              if (errno == EAGAIN || errno == EWOULDBLOCK)
                {
                  ;
                }
              else
                {
                  perror (NULL);
                  close (sfdn);
                  return (10);
                }
            }
          else if (recv_size == 0)
            {
              //
              // End of stream.
              //
              can_rx = 0;
              printf ("--end of receive stream--\n");
            }
          else
            {
              buffer[recv_size] = 0;
              printf ("%s", buffer);
             }
        }
      if (can_tx)
        {
          read_size = read (STDIN_FILENO, buffer, BUFSIZ);
          if (read_size < 0)
            {
              if (errno == EAGAIN || errno == EWOULDBLOCK)
                {
                  ;
                }
              else
                {
                  perror (NULL);
                  close (sfdn);
                  return (11);
                }
            }
          else
            {
              //
              // Verify the `stop' command.
              //
              if (strncmp (buffer, "stop\n", read_size) == 0)
                {
                  printf ("--closing send stream--\n");
                  can_tx = 0;
                }
              else
                {
                  //
                  //
                  //
                  sent_size = send (sfdn, &buffer,
                                    (size_t) read_size, 0);
                  if (sent_size < 0)
                    {
                      if (errno == EAGAIN || errno == EWOULDBLOCK)
                        {
                          ;
                        }
                      else
                        {
                          perror (NULL);
                          close (sfdn);
                          return (12);
                        }
                    }
                }
            }
        }
    }
  //
  // All done.
  //
  close (sfdn);
  return (0);
}
//----------------------------------------------------------------------
static void
usage (void)
{
  fprintf (stderr, "Usage: sc [-l] PATH\n");
}
