#include <unistd.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <errno.h>
#include <stdio.h>
#include <ctype.h>
#include <pwd.h>
//----------------------------------------------------------------------
static void     usage (void);
//----------------------------------------------------------------------
int
main (int argc, char *argv[], char *envp[])
{
  char           *user;
  int             uid;
  struct passwd  *pws;
  struct stat     file_status;
  int             a;		// Argument index.
  int             status;
  //
  //
  //
  if (argc < 3)
    {
      usage ();
      return (1);
    }
  //
  // Get user id number.
  //
  user = argv[1];
  if (isdigit (*user))
    {
      uid = atoi (user);
    }
  else
    {
      pws = getpwnam (user);
      if (pws == NULL)
	{
	  fprintf (stderr, "Unknown user \"%s\"!\n", user);
	  return (2);
	}
      uid = pws->pw_uid;
    }
  //
  // Now we have the user id. Start scanning file names.
  //
  for (a = 2; a < argc; a++)
    {
      //
      // Verify if the file exists, through the return value of
      // `stat()'. No other checks are made.
      //
      if (stat (argv[a], &file_status) == 0)
	{
	  //
	  // Try to change ownership.
	  //
	  status = chown (argv[a], uid, file_status.st_gid);
	  if (status != 0)
	    {
	      perror (NULL);
	      return (3);
	    }
	}
      else
	{
	  fprintf (stderr, "File \"%s\" does not exist!\n", argv[a]);
	  continue;
	}
    }
  //
  // All done.
  //
  return (0);
}

//----------------------------------------------------------------------
static void
usage (void)
{
  fprintf (stderr, "Usage:   chown USER|UID FILE...\n");
  fprintf (stderr, "Example: chown user my_file\n");
}
