#include <sys/os32.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <libgen.h>
//----------------------------------------------------------------------
static void     usage (void);
//----------------------------------------------------------------------
int
main (int argc, char *argv[], char *envp[])
{
  char           *source;
  char           *destination;
  char           *destination_full;
  struct stat     file_status;
  int             dest_is_a_dir = 0;
  int             a;		// Argument index.
  char            path[PATH_MAX];
  int             fd_source = -1;
  int             fd_destination = -1;
  char            buffer_in[BUFSIZ];
  char           *buffer_out;
  ssize_t         count_in;	// Read counter.
  ssize_t         count_out;	// Write counter.
  //
  // There must be at least two arguments, plus the program name.
  //
  if (argc < 3)
    {
      usage ();
      return (1);
    }
  //
  // Select the last argument as the destination.
  //
  destination = argv[argc - 1];
  //
  // Check if it is a directory and save it in a flag.
  //
  if (stat (destination, &file_status) == 0)
    {
      if (S_ISDIR (file_status.st_mode))
	{
	  dest_is_a_dir = 1;
	}
    }
  //
  // If there are more than two arguments, verify that the last
  // one is a directory.
  //
  if (argc > 3)
    {
      if (!dest_is_a_dir)
	{
	  usage ();
	  fprintf (stderr, "The destination \"%s\" ", destination);
	  fprintf (stderr, "is not a directory!\n");
	  return (1);
	}
    }
  //
  // Scan the arguments, excluded the last, that is the destination.
  //
  for (a = 1; a < (argc - 1); a++)
    {
      //
      // Source.
      //
      source = argv[a];
      //
      // Verify access permissions.
      //
      if (access (source, R_OK) < 0)
	{
	  perror (source);
	  continue;
	}
      //
      // Destination.
      //
      // If it is a directory, the destination path
      // must be corrected.
      //
      if (dest_is_a_dir)
	{
	  path[0] = 0;
	  strcat (path, destination);
	  strcat (path, "/");
	  strcat (path, basename (source));
	  //
	  // Update the destination path.
	  //
	  destination_full = path;
	}
      else
	{
	  destination_full = destination;
	}
      //
      // Check if destination file exists.
      //
      if (stat (destination_full, &file_status) == 0)
	{
	  fprintf (stderr, "The destination file, \"%s\", ",
		   destination_full);
	  fprintf (stderr, "already exists!\n");
	  continue;
	}
      //
      // Everything is ready for the copy.
      //
      fd_source = open (source, O_RDONLY);
      if (fd_source < 0)
	{
	  perror (source);
	  //
	  // Continue with the next file.
	  //
	  continue;
	}
      //
      fd_destination = creat (destination_full, 0777);
      if (fd_destination < 0)
	{
	  perror (destination);
	  close (fd_source);
	  //
	  // Continue with the next file.
	  //
	  continue;
	}
      //
      // Copy the data.
      //
      while (1)
	{
	  count_in = read (fd_source, buffer_in, (size_t) BUFSIZ);
	  if (count_in > 0)
	    {
	      for (buffer_out = buffer_in; count_in > 0;)
		{
		  count_out = write (fd_destination, buffer_out,
				     (size_t) count_in);
		  if (count_out < 0)
		    {
		      perror (destination);
		      close (fd_source);
		      close (fd_destination);
		      return (3);
		    }
		  //
		  // If not all data is written, continue writing,
		  // but change the buffer start position and the
		  // amount to be written.
		  //
		  buffer_out += count_out;
		  count_in -= count_out;
		}
	    }
	  else if (count_in < 0)
	    {
	      perror (source);
	      close (fd_source);
	      close (fd_destination);
	    }
	  else
	    {
	      break;
	    }
	}
      //
      if (close (fd_source))
	{
	  perror (source);
	}
      if (close (fd_destination))
	{
	  perror (destination);
	  return (4);
	}
    }
  //
  // All done.
  //
  return (0);
}

//----------------------------------------------------------------------
static void
usage (void)
{
  fprintf (stderr, "Usage: cp OLD_NAME NEW_NAME\n");
  fprintf (stderr, "       cp FILE... DIRECTORY\n");
}
