#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <libgen.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <stdint.h>
#include <stdbool.h>
//----------------------------------------------------------------------
#define DEBUG 0
static void usage (void);
static int send_file (int sfdn2, const char *path);
static int send_line (int sfdn2, const char *line);
char buffer[BUFSIZ];
char path_absolute[PATH_MAX];
//----------------------------------------------------------------------
int
main (int argc, char *argv[], char *envp[])
{
  int opt;
//extern char *optarg;              // not used.
  extern int optind;
  extern int optopt;
  //
  int status;
  int sfdn;
  int sfdn2;
  struct sockaddr_in sa_local;
  struct sockaddr_in sa_remote;
  socklen_t sa_remote_size = sizeof (struct sockaddr_in);
  ssize_t recv_size;
  char *addr = "0.0.0.0";
  char *www  = NULL;
  char *path  = NULL;
  int   port;
  bool request_read;
  int b;                    // index inside the buffer string buffer
  char *string = NULL;
  struct stat file_status;
  //
  // Check for options: no options at the moment.
  //
  while ((opt = getopt (argc, argv, ":")) != -1)
    {
      switch (opt)
        {
        case '?':
          fprintf (stderr, "Unknown option -%c.\n", optopt);
          usage ();
          return (1);
          break;
        case ':':
          fprintf (stderr, "Missing argument for option -%c\n", optopt);
          usage ();
          return (1);
          break;
        default:
          fprintf (stderr, "Getopt problem: unknown option %c\n", opt);
          usage ();
          return (1);
        }
    }
  //
  // Arguments.
  //
  if (optind == (argc - 2))
    {
      //
      // There are exactly two arguments: the port and the www root
      // path.
      //
      port = atoi (argv[argc - 2]);
      www = argv[argc - 1];
    }
  else
    {
      //
      // Arguments wrong!
      //
      printf ("optind = %i = %s, argc = %i\n", optind, argv[optind],
              argc);
      usage ();
      return (2);
    }
  //
  // Set the local address.
  //
  sa_local.sin_family = AF_INET;
  sa_local.sin_port = htons (port);
  inet_pton (AF_INET, addr, &sa_local.sin_addr.s_addr);
  //
  // Open the socket.
  //
  sfdn = socket (AF_INET, SOCK_STREAM, IPPROTO_TCP);
  if (sfdn < 0)
    {
      perror (NULL);
      return (3);
    }
  if (DEBUG)
    {
      printf ("HTTP: listening socket number is %i.\n", sfdn);
    }
  //
  // Set it listening: bind the local `sa' location.
  //
  status = bind (sfdn, (struct sockaddr *) &sa_local,
                 sizeof (sa_local));
  if (status < 0)
    {
      perror (NULL);
      close (sfdn);
      return (4);
    }
  //
  // Listen (TCP).
  //
  status = listen (sfdn, 1);
  if (status < 0)
    {
      perror (NULL);
      close (sfdn);
      return (5);
    }
  //
  // Accept connections, inside a loop.
  //
  while (1)
    {
      //
      // Accept.
      //
      if (DEBUG)
        {
          printf ("HTTP: listening socket number is %i.\n", sfdn);
        }
      //
      sfdn2 = accept (sfdn, (struct sockaddr *) &sa_remote,
                      &sa_remote_size);
      //
      if (sfdn2 < 0)
        {
          perror (NULL);
          close (sfdn2);
          continue;
        }
      if (DEBUG)
        {
          printf ("HTTP: new connection with socket number %i.\n",
                  sfdn2);
        }
      //
      // Define the socket non blocking.
      //
      status = fcntl (sfdn2, F_SETFL, O_NONBLOCK);
      if (status < 0)
        {
          perror (NULL);
          return (9);
        }
      //
      // Will read from the remote.
      //
      path_absolute[0] = 0;
      request_read = 1;
      while (request_read)
        {
          //
          // Read a line from the remote side.
          //
          for (b = 0; b < (BUFSIZ - 2); b++, buffer[b] = 0)
            {
              //
              // Read a single character from the remote side.
              //
              recv_size = recv (sfdn2, &buffer[b], (size_t) 1, 0);
              //
              if (recv_size < 0)
                {
                  if (errno == EAGAIN || errno == EWOULDBLOCK)
                    {
                      b--;
                      continue;
                    }
                  else
                    {
                      perror (NULL);
                      close (sfdn2);
                      continue;
                    }
                }
              if (recv_size == 0)
                {
                  //
                  // It is the end of stream, but should not happen.
                  //
                  buffer[b] = 0;
                  request_read = 0;
                  if (DEBUG)
                    {
                      printf ("HTTP: end of stream, "
                              "but should not happen here! \"%s\"\n",
                              buffer);
                    }
                  break;
                }
              //
              if (buffer[b] == '\r')
                {
                  //
                  // Ignore CR.
                  //
                  b--;
                  continue;
                }
              //
              if (buffer[b] == '\n')
                {
                  //
                  // End of line.
                  //
                  buffer[b] = 0;
                  if (DEBUG)
                    {
                      printf ("HTTP: %s\n", buffer);
                    }
                  break;
                }
            }
          //
          // Was it the end of the header?
          //
          if (strlen (buffer) == 0)
            {
              //
              // End of header.
              //
              request_read = 0;
              break;
            }
          //
          // We are reading the header: was it the GET command?
          //
          string = strtok (buffer, " ");
          if (strncmp (string, "GET", 4) == 0)
            {
              //
              // It is a GET: find the path.
              //
              path = strtok (NULL, " ");
              strncat (path_absolute, www, PATH_MAX-1);
              strncat (path_absolute, path,
                       (PATH_MAX - strlen (path_absolute) -1));
            }
        }
      //
      // Verify to have received a `GET' request.
      //
      if (strlen (path_absolute) == 0)
        {
          //
          // There is no path inside the GET command; maybe there is
          // no GET command either: 400
          //
          if (DEBUG)
            {
              printf ("HTTP: 400 Bad Request: "
                      "no path inside the GET command.\n");
            }
          send_line (sfdn2, "HTTP/1.0 400 Bad Request\r\n");
          send_line (sfdn2, "Content-Type: text/html\r\n");
          send_line (sfdn2, "Content-Length: 26\r\n");
          send_line (sfdn2, "\r\n");
          send_line (sfdn2, "<H1>400 Bad Request</H1>\r\n");
        }
      //
      // Verify the path.
      //
      if (stat (path_absolute, &file_status) != 0)
        {
          //
          // The path inside the GET command does not exists: 404
          //
          if (DEBUG)
            {
              printf ("HTTP: 404 Not Found: "
                      "the path \"%s\" does not exists.\n",
                      path_absolute);
            }
          send_line (sfdn2, "HTTP/1.0 404 Not Found\r\n");
          send_line (sfdn2, "Content-Type: text/html\r\n");
          send_line (sfdn2, "Content-Length: 24\r\n");
          send_line (sfdn2, "\r\n");
          send_line (sfdn2, "<H1>404 Not Found</H1>\r\n");
        }
      else
        {
          //
          // File exists: check the file type.
          //
          if (S_ISDIR (file_status.st_mode))
            {
              //
              // Test to find `index.html'.
              //
              strncat (path_absolute, "index.html",
                       (PATH_MAX - strlen (path_absolute) -1));
              //
              if (stat (path_absolute, &file_status) != 0)
                {
                  //
                  // The index file inside the path requested
                  // does not exists: 404
                  //
                  if (DEBUG)
                    {
                      printf ("HTTP: 404 Not Found: "
                              "the path \"%s\" does not exists.\n",
                               path_absolute);
                    }
                  send_line (sfdn2, "HTTP/1.0 404 Not Found\r\n");
                  send_line (sfdn2, "Content-Type: text/html\r\n");
                  send_line (sfdn2, "Content-Length: 24\r\n");
                  send_line (sfdn2, "\r\n");
                  send_line (sfdn2, "<H1>404 Not Found</H1>\r\n");
                }
	    }
          //
          // There is a file to send.
          //
          send_file (sfdn2, path_absolute);
        }
      //
      // The socket `sfdn2' might be already closed; if so, the
      // variable was reset to zero.
      //
      if (sfdn2 != 0) close (sfdn2);
      buffer[0] = 0;
      if (DEBUG)
        {
          printf ("HTTP: connection closed: continue listening.\n");
        }
      continue;
    }
  //
  // All done.
  //
  close (sfdn);
  return (0);
}
//----------------------------------------------------------------------
static int
send_file (int sfdn2, const char *path)
{
//  size_t sent_size;
  size_t file_size;
  struct stat file_status;
  char ascii_size[32];
  int fdn;
  char           *buffer_in = buffer;
  char           *buffer_out;
  ssize_t         count_in;	// Read counter.
  ssize_t         count_out;	// Write counter.
  //
  if (sfdn2 == 0) return (-1);
  //
  if (stat (path, &file_status) != 0)
    {
      perror (NULL);
      close (sfdn2);
      sfdn2 = 0;
      return (-1);
    }
  //
  file_size = file_status.st_size;
  sprintf (ascii_size, "%i", file_size);
  //
  fdn = open (path, O_RDONLY);
  if (fdn < 0)
    {
      if (DEBUG)
        {
          printf ("HTTP: 403 Forbidden: %s ", path);
        }
      perror (path);
      send_line (sfdn2, "HTTP/1.0 403 Forbidden\r\n");
      send_line (sfdn2, "Content-Type: text/html\r\n");
      send_line (sfdn2, "Content-Length: 24\r\n");
      send_line (sfdn2, "\r\n");
      send_line (sfdn2, "<H1>404 Forbidden</H1>\r\n");
      close (sfdn2);
      sfdn2 = 0;
      return (-1);
    }
  //
  send_line (sfdn2, "HTTP/1.0 200 OK\r\n");
  send_line (sfdn2, "Content-Type: text/html\r\n");
  send_line (sfdn2, "Content-Length: ");
  send_line (sfdn2, ascii_size);
  send_line (sfdn2, "\r\n");
  send_line (sfdn2, "\r\n");
  //
  // Copy the data.
  //
  while (1)
    {
      count_in = read (fdn, buffer_in, (size_t) BUFSIZ);
      if (count_in > 0)
        {
          for (buffer_out = buffer_in; count_in > 0;)
            {
              count_out = send (sfdn2, buffer_out,
                                 (size_t) count_in, 0);
              if (count_out < 0)
                {
                  if (errno == EAGAIN || errno == EWOULDBLOCK)
                    {
                      continue;
                    }
                  else
                    {
                      fprintf (stderr, "[HTTP] cannot send 1!\n");
                      perror (NULL);
                      close (fdn);
                      close (sfdn2);
                      sfdn2 = 0;
                      return (-1);
                    }
                }
              //
              // If not all data is written, continue writing,
              // but change the buffer start position and the
              // amount to be written.
              //
              buffer_out += count_out;
              count_in -= count_out;
            }
        }
      else if (count_in < 0)
        {
          perror (path);
          close (fdn);
          close (sfdn2);
          sfdn2 = 0;
          return (-1);
        }
      else
        {
          break;
        }
    }
  //
  return (0);
}

//----------------------------------------------------------------------
static int
send_line (int sfdn2, const char *line)
{
  size_t sent_size;
  size_t line_size = strlen (line);
  const char *start = line;
  //
  if (sfdn2 == 0) return (-1);
  //
  while (1)
    {
      errno = 0;
      sent_size = send (sfdn2, start, (size_t) line_size, 0);
      //
      //
      //
      if (DEBUG)
        {
          printf ("[HTTP] line_size=%i, sent_size=%i, error=%i\n",
                  (int) line_size, (int) sent_size, errno);
        }
      if (sent_size < 0)
        {
          if (errno == EAGAIN || errno == EWOULDBLOCK)
            {
              continue;
            }
          else
            {
              fprintf (stderr, "[HTTP] cannot send 2!\n");
              perror (NULL);
              close (sfdn2);
              sfdn2 = 0;
              return (-1);
            }
        }
      //
      //
      //
      if (sent_size < line_size)
        {
          start = &start[sent_size];
          line_size -= sent_size;
          //
          continue;
        }
      return (0);
    }
}
//----------------------------------------------------------------------
static void
usage (void)
{
  fprintf (stderr,
           "os32 http usage:\n"
           "\n"
           "http PORT WWW_ROOT_PATH\n"
           "\n"
           "PORT port number listening for connections\n"
           "WWW_ROOT_PATH root for the published documents.\n");
}
