#include <sys/os32.h>
#include <kernel/memory.h>
#include <kernel/proc.h>
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
//----------------------------------------------------------------------
uint32_t        mb_table[MEM_MAX_BLOCKS / 32];	// Memory blocks map.
unsigned int    mb_max = MEM_MAX_BLOCKS;	// Memory blocks max.
proc_t          process;
//----------------------------------------------------------------------
static int      mb_block_set0 (int block);
static void     mb_check (pid_t pid, addr_t address, size_t size);
static void     mb_residual (void);
//----------------------------------------------------------------------
int
main (int argc, char *argv[], char *envp[])
{
  int             i;
  int             fd;
  ssize_t         size_read;
  char           *buffer;
  pid_t           pid;
  proc_t         *ps;
  //
  // Get memory map.
  //
  fd = open ("/dev/kmem_map", O_RDONLY);
  if (fd < 0)
    {
      printf ("[%s] Cannot open \"/dev/kmem_map\" ", argv[0]);
      perror (NULL);
      return (0);
    }
  //
  buffer = (char *) mb_table;
  lseek (fd, (off_t) 0, SEEK_SET);
  for (i = 0; i < (MEM_MAX_BLOCKS / 8); i += size_read)
    {
      size_read = read (fd, &buffer[i], BUFSIZ);
      if (size_read < 0)
	{
	  printf ("[%s] Cannot read \"/dev/kmem_map\" %i %i  ",
		  argv[0], size_read, sizeof (mb_table));
	  perror (NULL);
	  return (0);
	}
    }
  //
  close (fd);
  //
  // Scan processes
  //
  buffer = (char *) &process;
  //
  fd = open ("/dev/kmem_ps", O_RDONLY);
  if (fd < 0)
    {
      printf ("[%s] Cannot open \"/dev/kmem_ps\" ", argv[0]);
      perror (NULL);
      exit (0);
    }
  //
  // Scan processes.
  //
  for (pid = 0; pid < PROCESS_MAX; pid++)
    {
      lseek (fd, (off_t) pid, SEEK_SET);
      size_read = read (fd, buffer, sizeof (proc_t));
      if (size_read < sizeof (proc_t))
	{
	  printf ("[%s] Cannot read \"/dev/kmem_ps\" pid %i ",
		  argv[0], pid);
	  perror (NULL);
	  continue;
	}
      ps = (proc_t *) buffer;
      if (ps->status > 0)
	{
	  //
	  //
	  //
	  if (ps->domain_data == 0)
	    {
	      mb_check (pid, ps->address_text,
			ps->domain_text + ps->extra_data);
	    }
	  else
	    {
	      mb_check (pid, ps->address_text, ps->domain_text);
	      mb_check (pid, ps->address_data,
			ps->domain_data + ps->extra_data);
	    }
	}
    }
  close (fd);
  //
  // Check residual allocation, if any.
  //
  mb_residual ();
  //
  return (0);
}

//----------------------------------------------------------------------
static void
mb_check (pid_t pid, addr_t address, size_t size)
{
  unsigned int    bstart;
  unsigned int    bsize;
  unsigned int    bend;
  unsigned int    i;
  addr_t          block_address;
  //
  //  k_printf ("releasing 0x%x, size 0x%x\n", (int) address,
  //            (int) size);
  //
  if (size == 0)
    {
      //
      // Zero means nothing.
      //
      return;
    }
  //
  if (size % MEM_BLOCK_SIZE)
    {
      bsize = size / MEM_BLOCK_SIZE + 1;
    }
  else
    {
      bsize = size / MEM_BLOCK_SIZE;
    }
  //
  bstart = address / MEM_BLOCK_SIZE;
  bend = bstart + bsize;
  //
  //
  //
  for (i = bstart; i < bend; i++)
    {
      if (mb_block_set0 (i))
	{
	  ;
	}
      else
	{
	  block_address = i;
	  block_address *= MEM_BLOCK_SIZE;
	  printf ("PID %i is using memory address 0x%x, "
		  "but it is reported free or already used by "
		  "another process!\n",
		  (int) pid, (unsigned int) block_address);
	}
    }
}

//----------------------------------------------------------------------
static int
mb_block_set0 (int block)
{
  int             i = block / 32;
  int             j = block % 32;
  uint32_t        mask = 0x80000000 >> j;
  if (mb_table[i] & mask)
    {
      mb_table[i] = mb_table[i] & ~mask;
      return (1);
    }
  else
    {
      return (0); // The block is already set to 0 inside the map!
    }
}

//----------------------------------------------------------------------
static void
mb_residual (void)
{
  unsigned int    block;
  unsigned int    blocks = MEM_MAX_BLOCKS;
  int             i;
  int             j;
  uint32_t        mask;
  unsigned int    start = 0;
  unsigned int    stop = 0;
  unsigned int    status = 0;
  //
  // Show residual allocated memory.
  //
  for (block = 0; block < blocks; block++)
    {
      i = block / 32;
      j = block % 32;
      mask = 0x80000000 >> j;
      if (mb_table[i] & mask)
	{
	  //
	  // Allocated block
	  //
	  if (status == 0)
	    {
	      status = 1;
	      start = block;
	    }
	}
      else
	{
	  //
	  // Not allocated block.
	  //
	  if (status == 1)
	    {
	      status = 0;
	      stop = block;
	    }
	}
      //
      //
      //
      if (stop > 0)
	{
	  printf ("residual allocation: %x-%x  ", start, stop);
	  start = 0;
	  stop = 0;
	}
    }
  printf ("\n");
}
