#include <unistd.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <signal.h>
#include <stdio.h>
#include <sys/wait.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <sys/os32.h>
//----------------------------------------------------------------------
#define PROMPT_SIZE     16
//----------------------------------------------------------------------
static void     sh_cd (int argc, char *argv[]);
static void     sh_pwd (int argc, char *argv[]);
static void     sh_umask (int argc, char *argv[]);
//----------------------------------------------------------------------
int
main (int argc, char *argv[], char *envp[])
{
  char   buffer_cmd[ARG_MAX / 2];
  char  *argv_cmd[ARG_MAX / 16];
//char   prompt[PROMPT_SIZE];
  uid_t  uid;
  int    argc_cmd;
  pid_t  pid_cmd;
  pid_t  pid_dead;
  int    status;
  void  *pstatus;
  int    i;
  //
  //
  //
  uid = geteuid ();
  //
  // Load processes, reading the keyboard.
  //
  while (1)
    {
      if (uid == 0)
	{
	  printf ("# ");
	}
      else
	{
	  printf ("$ ");
	}
      //
      pstatus = fgets (buffer_cmd, (ARG_MAX / 2), stdin);
      if (pstatus == NULL)
        {
          if (errno)
            {
              perror (NULL);
              continue;
            }
          else
            {
              //
              // End of file, like ^D.
              //
              return (0);
            }
        }
      //
      i = strlen (buffer_cmd);
      if (i > 0 && buffer_cmd[i-1] == '\n')
        {
          buffer_cmd[i-1] = '\0';
        }
      //
      // Clear `argv_cmd[]';
      //
      for (argc_cmd = 0; argc_cmd < (ARG_MAX / 16); argc_cmd++)
	{
	  argv_cmd[argc_cmd] = NULL;
	}
      //
      // Initialize the command scan.
      //
      argv_cmd[0] = strtok (buffer_cmd, " \t");
      //
      // Verify: if the input is not valid, loop again.
      //
      if (argv_cmd[0] == NULL)
	{
	  continue;
	}
      //
      // Find the arguments.
      //
      for (argc_cmd = 1;
	   argc_cmd < ((ARG_MAX / 16) - 1)
	   && argv_cmd[argc_cmd - 1] != NULL; argc_cmd++)
	{
	  argv_cmd[argc_cmd] = strtok (NULL, " \t");
	}
      //
      // If there are too many arguments, show a message and continue.
      //
      if (argv_cmd[argc_cmd - 1] != NULL)
	{
	  errset (E2BIG);	// Argument list too long.
	  perror (NULL);
	  continue;
	}
      //
      // Correct the value for `argc_cmd', because actually
      // it counts also the NULL element.
      //
      argc_cmd--;
      //
      // Verify if it is an internal command.
      //
      if (strcmp (argv_cmd[0], "exit") == 0)
	{
	  return (0);
	}
      else if (strcmp (argv_cmd[0], "cd") == 0)
	{
	  sh_cd (argc_cmd, argv_cmd);
	  continue;
	}
      else if (strcmp (argv_cmd[0], "pwd") == 0)
	{
	  sh_pwd (argc_cmd, argv_cmd);
	  continue;
	}
      else if (strcmp (argv_cmd[0], "umask") == 0)
	{
	  sh_umask (argc_cmd, argv_cmd);
	  continue;
	}
      //
      // It should be a program to run.
      //
      pid_cmd = fork ();
      if (pid_cmd == -1)
	{
	  printf ("%s: cannot run command", argv[0]);
	  perror (NULL);
	}
      else if (pid_cmd == 0)
	{
	  execvp (argv_cmd[0], argv_cmd);
	  perror (NULL);
	  exit (0);
	}
      while (1)
	{
	  pid_dead = wait (&status);
	  if (pid_dead == pid_cmd)
	    {
	      break;
	    }
	}
      printf ("pid %i terminated with status %i.\n",
	      (int) pid_dead, status);
    }
}

//----------------------------------------------------------------------
static void
sh_cd (int argc, char *argv[])
{
  int             status;
  //
  if (argc != 2)
    {
      errset (EINVAL);		// Invalid argument.
      perror (NULL);
      return;
    }
  //
  status = chdir (argv[1]);
  if (status != 0)
    {
      perror (NULL);
    }
  return;
}

//----------------------------------------------------------------------
static void
sh_pwd (int argc, char *argv[])
{
  char            path[PATH_MAX];
  void           *pstatus;
  //
  if (argc != 1)
    {
      errset (EINVAL);		// Invalid argument.
      perror (NULL);
      return;
    }
  //
  // Get the current directory.
  //
  pstatus = getcwd (path, (size_t) PATH_MAX);
  if (pstatus == NULL)
    {
      perror (NULL);
    }
  else
    {
      printf ("%s\n", path);
    }
  return;
}

//----------------------------------------------------------------------
static void
sh_umask (int argc, char *argv[])
{
  sysmsg_uarea_t  msg;
  char           *m;		// Index inside the umask octal string.
  int             mask;
  int             digit;
  //
  if (argc > 2)
    {
      errset (EINVAL);		// Invalid argument.
      perror (NULL);
      return;
    }
  //
  // If no argument is available, the umask is shown, with a direct
  // system call.
  //
  if (argc == 1)
    {
      sys (SYS_UAREA, &msg, (sizeof msg));
      printf ("%04o\n", msg.umask);
      return;
    }
  //
  // Get the mask: must be the first argument.
  //
  for (mask = 0, m = argv[1]; *m != 0; m++)
    {
      digit = (*m - '0');
      if (digit < 0 || digit > 7)
	{
	  errset (EINVAL);	// Invalid argument.
	  perror (NULL);
	  return;
	}
      mask = mask * 8 + digit;
    }
  //
  // Set the umask and return.
  //
  umask (mask);
  return;
}
