#include <kernel/driver/ata.h>
#include <kernel/lib_k.h>
#include <kernel/ibm_i386.h>
#include <stdint.h>
#include <errno.h>
//----------------------------------------------------------------------
int
ata_cmd_write_sectors (int drive, unsigned int sector,
                       unsigned char count, void *buffer)
{
    int       s;
    int       i;
    int       c;
    uint16_t *source = buffer;
    //
    // Set LBA 28 parameters.
    //
    s = ata_lba28 (drive, sector, count);
    if (s < 0)
      {
        errset (errno);
        return (-1);
      }
    //
    // Send `command'
    //
    out_8 (ata_table[drive].r_command, ATA_COMMAND_WRITE_SECTORS);
    //
    // Parameter `count' equal to zero means 256 sectors.
    //
    if (count == 0)
      {
        c = 256;
      }
    else
      {
        c = count;
      }
    //
    // Read `c' sectors.
    //
    for (; c > 0; c--)
      {
        s = ata_drq (drive, ATA_TIMEOUT);
        if (s < 0)
          {
            errset (errno);
            return (-1);
          }
        //
        // Write sector.
        //
        for (i = 0; i < (ATA_SECTOR_SIZE/2); i++)
          {
            out_16 (ata_table[drive].r_data, source[i]);
          }
      }
    //
    // Wait that the device returns ready.
    //
    s = ata_rdy (drive, ATA_TIMEOUT);
    if (s < 0)
      {
        errset (errno);
        return (-1);
      }
    //
    // Now flush cache.
    //
    out_8 (ata_table[drive].r_command, ATA_COMMAND_FLUSH_CACHE);
    //
    // Then wait that the device returns ready.
    //
    s = ata_rdy (drive, ATA_TIMEOUT_FLUSH);
    if (s < 0)
      {
        errset (errno);
        return (-1);
      }
    //
    // Return.
    //
    return (0);
}
