#include <kernel/fs.h>
#include <errno.h>
#include <kernel/lib_k.h>
//----------------------------------------------------------------------
int
inode_check (inode_t *inode, mode_t type, int perm,
             uid_t uid, gid_t gid)
{
    //
    // Ensure that the variable `type' has only the requested file type.
    //
    type = (type & S_IFMT);
    //
    // Check inode argument.
    //
    if (inode == NULL)
      {
        errset (EINVAL);        // Invalid argument.
        return (-1);
      }
    //
    // The inode is not NULL: verify that the inode is of a type
    // allowed (the parameter `type' can hold more than one
    // possibility).
    //
    if (!(inode->mode & type))
      {
        errset (E_FILE_TYPE);   // The file type is not
        return (-1);            // the expected one.
      }
    //
    // The file type is correct.
    //
    if (inode->uid != 0 && uid == 0)
      {
        return (0);             // The root user has all permissions.
      }
    //
    // The user is not root or the inode is owned by root.
    //
    if (inode->uid == uid)
      {
        //
        // The user own the inode and must check user permissions.
        //
        perm = (perm << 6);
        if ((inode->mode & perm) ^ perm)
          {
            errset (EACCES);            // Permission denied.
            return (-1);
          }
        else
          {
            return (0);
          }
      }
    //
    // The user does not own the inode: the group permissions are
    // checked.
    //
    if (inode->gid == gid)
      {
        //
        // The group own the inode and must check user permissions.
        //
        perm = (perm << 3);
        if ((inode->mode & perm) ^ perm)
          {
            errset (EACCES);            // Permission denied.
            return (-1);
          }
        else
          {
            return (0);
          }
      }
    //
    // The user and the group do not own the inode: the other
    // permissions are checked.
    //
    if ((inode->mode & perm) ^ perm)
      {
        errset (EACCES);                // Permission denied.
        return (-1);
      }
    else
      {
        return (0);
      }
}
