#include <kernel/fs.h>
#include <errno.h>
//----------------------------------------------------------------------
ssize_t
inode_pipe_write (inode_t *inode, const void *buffer, size_t count)
{
    const unsigned char *buffer_s = buffer;
    unsigned char *buffer_d;
    int            i;
    //
    // The inode pointer must be valid.
    //
    if (inode == NULL)
      {
        errset (EINVAL);        // Invalid argument.
        return ((ssize_t) -1);
      }
    //
    // Check the current pipe direction and see if can be
    // written something.
    //
    if (inode->pipe_dir)
      {
        //
        // Write: the pipe is waiting for a write.
        //
        ;
      }
    else
      {
        //
        // Read: if indexes are the same, cannot write anything.
        //
        if (inode->pipe_off_write == inode->pipe_off_read)
          {
            //
            // Cannot write. More checks will be made by `s_write()'.
            //
            return ((ssize_t) 0);
          }
      }
    //
    // Might write something. Set the pointer to the destination buffer,
    // that is the area used for direct zones, including first indirect
    // pointers (total: (7+2)*2 = 18 bytes).
    //
    buffer_d = (void *) &(inode->direct[0]);
    //
    i = 0;
    //
    if (inode->pipe_off_write >= inode->pipe_off_read)
      {
        for (; i < count; i++)
          {
            if (inode->pipe_off_write < INODE_PIPE_BUFFER_SIZE)
              {
                buffer_d[inode->pipe_off_write] = buffer_s[i];
                inode->pipe_off_write++;
              }
            else
              {
                inode->pipe_off_write = 0;
                break;
              }
          }
      }
    //
    if (inode->pipe_off_write < inode->pipe_off_read)
      {
        for (; i < count; i++)
          {
            if (inode->pipe_off_write < inode->pipe_off_read)
              {
                buffer_d[inode->pipe_off_write] = buffer_s[i];
                inode->pipe_off_write++;
              }
            else
              {
                break;
              }
          }
      }
    //
    // At this point, it is time to set the direction to read;
    // it doesn't matter if the direction is already set so.
    //
    if (inode->pipe_off_write == inode->pipe_off_read)
      {
        inode->pipe_dir = 0;
      }
    //
    // Ok.
    //
    return ((ssize_t) i);
}
