#include <sys/os32.h>
#include <kernel/fs.h>
#include <kernel/lib_k.h>
#include <time.h>
//----------------------------------------------------------------------
void
inode_print (void)
{
    int        i;
    dev_t      device_attached = 0;
    time_t     time;
    struct tm *timeptr;
    char       type;
    dev_t      device;
    //
    k_printf (" dev   ino ref c mntd t mode  uid gid size Kib "
              "date       time     lnk dirct[0]\n");
    //
    for (i = 0; i < INODE_MAX_SLOTS; i++)
      {
        if (inode_table[i].references <= 0)
          {
            continue;
          }
        //
        // Calculate modification time.
        //
        time = inode_table[i].time;
        //
        timeptr = gmtime (&time);
        //
        // Get type from mode.
        //
        if      (S_ISBLK  (inode_table[i].mode)) type = 'b';
        else if (S_ISCHR  (inode_table[i].mode)) type = 'c';
        else if (S_ISFIFO (inode_table[i].mode)) type = 'p';
        else if (S_ISREG  (inode_table[i].mode)) type = '-';
        else if (S_ISDIR  (inode_table[i].mode)) type = 'd';
        else if (S_ISLNK  (inode_table[i].mode)) type = 'l';
        else if (S_ISSOCK (inode_table[i].mode)) type = 's';
        else                                     type = '?';
        //
        // Is it a mount point?
        //
        if (inode_table[i].sb_attached != NULL)
          {
            device_attached = inode_table[i].sb_attached->device;
          }
        //
        // Is there a super block device?
        //
        if (inode_table[i].sb == NULL)
          {
            device = 0;
          }
        else
          {
            device = inode_table[i].sb->device;
          }
        //
        // Print data.
        //
        k_printf ("%04x %5i %3i %c %04x %c %04o %4i %3i %8i "
                  "%4i.%02i.%02i %2i:%02i:%02i %3i %08x\n",
                  (unsigned int) device,
                  (unsigned int) inode_table[i].ino,
                  (unsigned int) inode_table[i].references,
                  (inode_table[i].changed ? '!' : ' '),
                  (unsigned int) device_attached,
                  type,
                  (unsigned int) inode_table[i].mode,
                  (unsigned int) inode_table[i].uid,
                  (unsigned int) inode_table[i].gid,
                  (unsigned int) (inode_table[i].size / 1024),
                  timeptr->tm_year,
                  timeptr->tm_mon,
                  timeptr->tm_mday,
                  timeptr->tm_hour,
                  timeptr->tm_min,
                  timeptr->tm_sec,
                  (unsigned int) inode_table[i].links,
                  (unsigned int) inode_table[i].direct[0]);
      }
}
