#include <kernel/ibm_i386.h>
//----------------------------------------------------------------------
void
idt (void)
{
    //
    // Set necessary data for the IDTR register.
    //
    idt_register.limit = (sizeof (idt_table) - 1);
    idt_register.base  = (uint32_t) &idt_table[0];
    //
    // Reset all items inside the array `idt_table[]'.
    //
    int i;
    for (i = 0; i < IDT_ITEMS; i++)
      {
        idt_descriptor (i, 0, 0, 0, 0, 0);
      }
    //
    // Place hardware interrupt from IRQ 0 to IRQ 7
    // starting from descriptor 32 and from IRQ 8 to
    // IRQ 15 starting from descriptor 40.
    //
    idt_irq_remap (32, 40);
    //
    // Set the ISR routines to the items inside the IDT table.
    //
    idt_descriptor ( 0, isr_0,  0x0008, 1, 0xE, 0);
    idt_descriptor ( 1, isr_1,  0x0008, 1, 0xE, 0);
    idt_descriptor ( 2, isr_2,  0x0008, 1, 0xE, 0);
    idt_descriptor ( 3, isr_3,  0x0008, 1, 0xE, 0);
    idt_descriptor ( 4, isr_4,  0x0008, 1, 0xE, 0);
    idt_descriptor ( 5, isr_5,  0x0008, 1, 0xE, 0);
    idt_descriptor ( 6, isr_6,  0x0008, 1, 0xE, 0);
    idt_descriptor ( 7, isr_7,  0x0008, 1, 0xE, 0);
    idt_descriptor ( 8, isr_8,  0x0008, 1, 0xE, 0);
    idt_descriptor ( 9, isr_9,  0x0008, 1, 0xE, 0);
    idt_descriptor (10, isr_10, 0x0008, 1, 0xE, 0);
    idt_descriptor (11, isr_11, 0x0008, 1, 0xE, 0);
    idt_descriptor (12, isr_12, 0x0008, 1, 0xE, 0);
    idt_descriptor (13, isr_13, 0x0008, 1, 0xE, 0);
    idt_descriptor (14, isr_14, 0x0008, 1, 0xE, 0);
    idt_descriptor (15, isr_15, 0x0008, 1, 0xE, 0);
    idt_descriptor (16, isr_16, 0x0008, 1, 0xE, 0);
    idt_descriptor (17, isr_17, 0x0008, 1, 0xE, 0);
    idt_descriptor (18, isr_18, 0x0008, 1, 0xE, 0);
    idt_descriptor (19, isr_19, 0x0008, 1, 0xE, 0);
    idt_descriptor (20, isr_20, 0x0008, 1, 0xE, 0);
    idt_descriptor (21, isr_21, 0x0008, 1, 0xE, 0);
    idt_descriptor (22, isr_22, 0x0008, 1, 0xE, 0);
    idt_descriptor (23, isr_23, 0x0008, 1, 0xE, 0);
    idt_descriptor (24, isr_24, 0x0008, 1, 0xE, 0);
    idt_descriptor (25, isr_25, 0x0008, 1, 0xE, 0);
    idt_descriptor (26, isr_26, 0x0008, 1, 0xE, 0);
    idt_descriptor (27, isr_27, 0x0008, 1, 0xE, 0);
    idt_descriptor (28, isr_28, 0x0008, 1, 0xE, 0);
    idt_descriptor (29, isr_29, 0x0008, 1, 0xE, 0);
    idt_descriptor (30, isr_10, 0x0008, 1, 0xE, 0);
    idt_descriptor (31, isr_31, 0x0008, 1, 0xE, 0);
    idt_descriptor (32, isr_32, 0x0008, 1, 0xE, 0);
    idt_descriptor (33, isr_33, 0x0008, 1, 0xE, 0);
    idt_descriptor (34, isr_34, 0x0008, 1, 0xE, 0);
    idt_descriptor (35, isr_35, 0x0008, 1, 0xE, 0);
    idt_descriptor (36, isr_36, 0x0008, 1, 0xE, 0);
    idt_descriptor (37, isr_37, 0x0008, 1, 0xE, 0);
    idt_descriptor (38, isr_38, 0x0008, 1, 0xE, 0);
    idt_descriptor (39, isr_39, 0x0008, 1, 0xE, 0);
    idt_descriptor (40, isr_40, 0x0008, 1, 0xE, 0);
    idt_descriptor (41, isr_41, 0x0008, 1, 0xE, 0);
    idt_descriptor (42, isr_42, 0x0008, 1, 0xE, 0);
    idt_descriptor (43, isr_43, 0x0008, 1, 0xE, 0);
    idt_descriptor (44, isr_44, 0x0008, 1, 0xE, 0);
    idt_descriptor (45, isr_45, 0x0008, 1, 0xE, 0);
    idt_descriptor (46, isr_46, 0x0008, 1, 0xE, 0);
    idt_descriptor (47, isr_47, 0x0008, 1, 0xE, 0);
    //
    // The following item is for the system calls.
    //
    idt_descriptor (128, isr_128, 0x0008, 1, 0xE, 0);
    //
    // Activate the IDT loading the IDTR register.
    //
    idt_load (&idt_register);
}
