#include <kernel/fs.h>
#include <errno.h>
#include <kernel/proc.h>
#include <kernel/lib_s.h>
//----------------------------------------------------------------------
int
s_chown (pid_t pid, const char *path, uid_t uid, gid_t gid)
{
  proc_t         *ps;
  inode_t        *inode;
  //
  // Get process.
  //
  ps = proc_reference (pid);
  //
  // Must be root, as the ability to change group is not considered.
  //
  if (ps->euid != 0)
    {
      errset (EPERM);		// Operation not permitted.
      return (-1);
    }
  //
  // Try to load the file inode.
  //
  inode = path_inode (pid, path);
  if (inode == NULL)
    {
      //
      // Cannot access the file: it does not exists or permissions are
      // not sufficient. Variable `errno' is set by function
      // `inode_directory()'.
      //
      return (-1);
    }
  //
  // Update the owner and group.
  //
  if (uid != -1)
    {
      inode->uid = uid;
      inode->changed = 1;
    }
  if (gid != -1)
    {
      inode->gid = gid;
      inode->changed = 1;
    }
  //
  // Save and release the inode.
  //
  inode_save (inode);
  inode_put (inode);
  //
  // Return.
  //
  return (0);
}
