#include <kernel/proc.h>
#include <fcntl.h>
#include <errno.h>
//----------------------------------------------------------------------
int
s_close (pid_t pid, int fdn)
{
  fd_t *fd;
  int   status;
  //
  // Get file descriptor.
  //
  fd = fd_reference (pid, &fdn);
  if (fd == NULL || fd->file == NULL
      || (fd->file->inode == NULL && fd->file->sock == NULL))
    {
      errset (EBADF);           // Bad file descriptor.
      return (-1);
    }
  //
  //
  //
  if (fd->file->inode != NULL)                                  // Inode
    {
      //
      // File descriptor with inode.
      //
      // If it is a pipe, some special things must be done.
      //
      if (S_ISFIFO (fd->file->inode->mode))
        {
          if (fd->fl_flags & O_RDONLY)
            {
              fd->file->inode->pipe_ref_read--;
              if (fd->file->inode->pipe_ref_read == 0)
                {
                  proc_wakeup_pipe_write (fd->file->inode);
                }
            }
          //
          if (fd->fl_flags & O_WRONLY)
            {
              fd->file->inode->pipe_ref_write--;
              if (fd->file->inode->pipe_ref_write == 0)
                {
                  proc_wakeup_pipe_read (fd->file->inode);
                }
            }
        }
    }
  else                                                         // Socket
    {
      //
      // File descriptor with socket.
      //
      status = tcp_close (fd->file->sock);
      if (status < 0 && (errno == EINPROGRESS || errno == EALREADY))
        {
          errset (errno);
          return (status);
        }
      //
      // Otherwise, the socket is closed and can continue
      // with the other references.
      //
    }
  //
  // Reduce references inside the file table item
  // and remove item if it reaches zero.
  //
  fd->file->references--;
  if (fd->file->references == 0)
    {
      fd->file->oflags = 0;
      fd->file->inode = NULL;
      //
      // Put inode, or release the socket, because there are no more
      // file references.
      //
      if (fd->file->inode != NULL)
        {
          inode_put (fd->file->inode);
        }
      if (fd->file->sock != NULL)
        {
          sock_put (fd->file->sock);
        }
    }
  //
  // Remove file descriptor.
  //
  fd->fl_flags = 0;
  fd->fd_flags = 0;
  fd->file = NULL;
  //
  //
  //
  return (0);
}
