#include <kernel/proc.h>
#include <kernel/lib_s.h>
#include <errno.h>
#include <fcntl.h>
//----------------------------------------------------------------------
int
s_fstat (pid_t pid, int fdn, struct stat *buffer)
{
  proc_t         *ps;
  inode_t        *inode;
  //
  // Get process.
  //
  ps = proc_reference (pid);
  //
  // Verify if the file descriptor is valid.
  //
  if (ps->fd[fdn].file == NULL)
    {
      errset (EBADF);		// Bad file descriptor.
      return (-1);
    }
  //
  // Reach the inode.
  //
  inode = ps->fd[fdn].file->inode;
  //
  // If the Inode does not exist, exit with error.
  //
  if (inode == NULL)
    {
      errset (ENOENT);
      return (-1);
    }
  //
  // Inode loaded: update the buffer.
  //
  buffer->st_dev = inode->sb->device;
  buffer->st_ino = inode->ino;
  buffer->st_mode = inode->mode;
  buffer->st_nlink = inode->links;
  buffer->st_uid = inode->uid;
  buffer->st_gid = inode->gid;
  if (S_ISBLK (buffer->st_mode) || S_ISCHR (buffer->st_mode))
    {
      buffer->st_rdev = inode->direct[0];
    }
  else
    {
      buffer->st_rdev = 0;
    }
  buffer->st_size = inode->size;
  buffer->st_atime = inode->time;	// All times are the same for
  buffer->st_mtime = inode->time;	// Minix 1 file system.
  buffer->st_ctime = inode->time;	//
  buffer->st_blksize = inode->sb->blksize;
  buffer->st_blocks = inode->blkcnt;
  //
  // If the inode is a device special file, the `st_rdev' value is
  // taken from the first direct zone (as of Minix 1 organization).
  //
  if (S_ISBLK (inode->mode) || S_ISCHR (inode->mode))
    {
      buffer->st_rdev = inode->direct[0];
    }
  else
    {
      buffer->st_rdev = 0;
    }
  //
  // Return.
  //
  return (0);
}
