#include <kernel/proc.h>
#include <kernel/lib_s.h>
#include <kernel/lib_k.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <arpa/inet.h>
//----------------------------------------------------------------------
int
s_listen (pid_t pid, int sfdn, int backlog)
{
  fd_t *sfd;
  int   s;
  //
  // Get file descriptor and verify that it is a socket.
  //
  sfd = fd_reference (pid, &sfdn);
  if (sfd == NULL || sfd->file == NULL)
    {
      errset (EBADF);		// Bad file descriptor.
      return (-1);
    }
  if (sfd->file->sock == NULL)
    {
      errset (ENOTSOCK);	// Not a socket.
      return (-1);
    }
  if (sfd->file->sock->type != SOCK_STREAM)
    {
      errset (EOPNOTSUPP);	// Not a stream socket.
      return (-1);
    }
  if (sfd->file->sock->raddr != 0
      || sfd->file->sock->rport != 0)
    {
      //
      // The socket is connected, and cannot be good for
      // listening.
      //
      errset (EISCONN);
      return (-1);
    }
  //
  // Scan the other sockets to find if there is another one listening.
  //
  for (s = 0; s < SOCK_MAX_SLOTS; s++)
    {
      if (sock_table[s].tcp.conn == TCP_LISTEN
          && sock_table[s].lport == sfd->file->sock->lport)
        {
          //
          // Yes, there is one: sorry.
          //
          errset (EADDRINUSE);
          return (-1);
        }
    }
  //
  // Check the current TCP state.
  //
  if (sfd->file->sock->tcp.conn != 0
      && sfd->file->sock->tcp.conn != TCP_LISTEN)
    {
      //
      // Cannot change the socket stream state.
      //
      errset (EISCONN);
      return (-1);
    }
  //
  // The socket might be already listening, but the newly requested
  // queue should be greater or equal to the previous one.
  //
  if (sfd->file->sock->tcp.conn == TCP_LISTEN
      && backlog < sfd->file->sock->tcp.listen_max)
    {
      //
      // Cannot reduce the listen queue: just ignore.
      //
      return (0);
    }
  //
  // Ok.
  //
  sfd->file->sock->tcp.conn = TCP_LISTEN;
  sfd->file->sock->tcp.listen_max = min (backlog, SOCK_MAX_QUEUE);
  sfd->file->sock->tcp.listen_pid = pid;
  return (0);
}
