#include <kernel/fs.h>
#include <errno.h>
#include <kernel/proc.h>
#include <kernel/lib_s.h>
//----------------------------------------------------------------------
int
s_mknod (pid_t pid, const char *path, mode_t mode, dev_t device)
{
  proc_t         *ps;
  inode_t        *inode;
  char            full_path[PATH_MAX];
  //
  // Get process.
  //
  ps = proc_reference (pid);
  //
  // Correct the mode with the umask.
  //
  mode &= ~ps->umask;
  //
  // Currently must be root, unless the type is a regular file,
  // or a FIFO file.
  //
  if (!(S_ISFIFO (mode) || S_ISREG (mode)))
    {
      if (ps->uid != 0)
	{
	  errset (EPERM);	// Operation not permitted.
	  return (-1);
	}
    }
  //
  // Check the type of node requested.
  //
  if (!(S_ISBLK (mode) ||
	S_ISCHR (mode) ||
	S_ISREG (mode) || S_ISFIFO (mode) || S_ISDIR (mode)))
    {
      errset (EINVAL);		// Invalid argument.
      return (-1);
    }
  //
  // Check if something already exists with the same name.
  //
  inode = path_inode (pid, path);
  if (inode != NULL)
    {
      //
      // The file already exists. Put inode and return an error.
      //
      inode_put (inode);
      errset (EEXIST);		// File exists.
      return (-1);
    }
  //
  // Try to creat the node.
  //
  path_full (path, ps->path_cwd, full_path);
  inode = path_inode_link (pid, full_path, NULL, mode);
  if (inode == NULL)
    {
      //
      // Sorry: cannot create the inode!
      //
      return (-1);
    }
  //
  // Set the device number if necessary.
  //
  if (S_ISBLK (mode) || S_ISCHR (mode))
    {
      inode->direct[0] = device;
      inode->changed = 1;
    }
  //
  // Put the inode.
  //
  inode_put (inode);
  //
  // Return.
  //
  return (0);
}
