#include <kernel/fs.h>
#include <errno.h>
#include <kernel/proc.h>
#include <kernel/lib_s.h>
//----------------------------------------------------------------------
int
s_mount (pid_t pid, const char *path_dev, const char *path_mnt,
	 int options)
{
  proc_t         *ps;
  dev_t           device;	// Device to mount.
  inode_t        *inode_mnt;	// Directory mount point.
  void           *pstatus;
  //
  // Get process.
  //
  ps = proc_reference (pid);
  //
  // Verify to be the super user.
  //
  if (ps->euid != 0)
    {
      errset (EPERM);		// Operation not permitted.
      return (-1);
    }
  //
  device = path_device (pid, path_dev);
  if (device < 0)
    {
      return (-1);
    }
  //
  inode_mnt = path_inode (pid, path_mnt);
  if (inode_mnt == NULL)
    {
      return (-1);
    }
  if (!S_ISDIR (inode_mnt->mode))
    {
      inode_put (inode_mnt);
      errset (ENOTDIR);		// Not a directory.
      return (-1);
    }
  if (inode_mnt->sb_attached != NULL)
    {
      inode_put (inode_mnt);
      errset (EBUSY);		// Device or resource busy.
      return (-1);
    }
  //
  // All data is available.
  //
  pstatus = sb_mount (device, &inode_mnt, options);
  if (pstatus == NULL)
    {
      inode_put (inode_mnt);
      return (-1);
    }
  //
  return (0);
}
