#include <kernel/fs.h>
#include <errno.h>
#include <kernel/proc.h>
#include <kernel/lib_s.h>
//----------------------------------------------------------------------
int
s_stat (pid_t pid, const char *path, struct stat *buffer)
{
  proc_t         *ps;
  inode_t        *inode;
  //
  // Check path.
  //
  if (path == NULL || strlen (path) == 0)
    {
      errset (EINVAL);
      return (-1);
    }
  //
  // Get process.
  //
  ps = proc_reference (pid);
  //
  // Try to load the file inode.
  //
  inode = path_inode (pid, path);
  if (inode == NULL)
    {
      //
      // Cannot access the file: it does not exists or permissions are
      // not sufficient. Variable `errno' is set by function
      // `path_inode()'.
      //
      errset (errno);
      return (-1);
    }
  //
  // Inode loaded: update the buffer.
  //
  buffer->st_dev = inode->sb->device;
  buffer->st_ino = inode->ino;
  buffer->st_mode = inode->mode;
  buffer->st_nlink = inode->links;
  buffer->st_uid = inode->uid;
  buffer->st_gid = inode->gid;
  if (S_ISBLK (buffer->st_mode) || S_ISCHR (buffer->st_mode))
    {
      buffer->st_rdev = inode->direct[0];
    }
  else
    {
      buffer->st_rdev = 0;
    }
  buffer->st_size = inode->size;
  buffer->st_atime = inode->time;	// All times are the same for
  buffer->st_mtime = inode->time;	// Minix 1 file system.
  buffer->st_ctime = inode->time;	//
  buffer->st_blksize = inode->sb->blksize;
  buffer->st_blocks = inode->blkcnt;
  //
  // If the inode is a device special file, the `st_rdev' value is
  // taken from the first direct zone (as of Minix 1 organization).
  //
  if (S_ISBLK (inode->mode) || S_ISCHR (inode->mode))
    {
      buffer->st_rdev = inode->direct[0];
    }
  else
    {
      buffer->st_rdev = 0;
    }
  //
  // Release the inode and return.
  //
  inode_put (inode);
  //
  // Return.
  //
  return (0);
}
