#include <kernel/fs.h>
#include <errno.h>
#include <kernel/proc.h>
#include <kernel/lib_s.h>
#include <termios.h>
#include <sys/types.h>
//----------------------------------------------------------------------
int
s_tcgetattr (pid_t pid, int fdn, struct termios *termios_p)
{
  file_t         *file;
  inode_t        *inode;
  dev_t           device;
  int             t;		// `tty_table[]' subscript.
  int             c;		// `c_cc[]' subscript.
  //
  file = proc_table[pid].fd[fdn].file;
  //
  if (file == NULL)
    {
      errset (EBADF);
      return (-1);
    }
  //
  inode = proc_table[pid].fd[fdn].file->inode;
  //
  if (inode == NULL)
    {
      errset (EBADF);
      return (-1);
    }
  //
  if (!S_ISCHR (inode->mode))
    {
      errset (ENOTTY);
      return (-1);
    }
  //
  device = inode->direct[0];
  //
  if (major (device) != DEV_CONSOLE_MAJOR)
    {
      errset (ENOTTY);
      return (-1);
    }
  //
  t = minor (device);
  if (t >= TTYS_TOTAL)
    {
      errset (ENOTTY);
      return (-1);
    }
  //
  // Ok: copy data.
  //
  termios_p->c_iflag = tty_table[t].attr.c_iflag;
  termios_p->c_oflag = tty_table[t].attr.c_oflag;
  termios_p->c_cflag = tty_table[t].attr.c_cflag;
  termios_p->c_lflag = tty_table[t].attr.c_lflag;
  for (c = 0; c < NCCS; c++)
    {
      termios_p->c_cc[c] = tty_table[t].attr.c_cc[c];
    }
  //
  // Ok.
  //
  return (0);
}
