.extern kmain
.extern _k_stack_top
.extern _k_stack_bottom
.global kstartup
########################################################################
#
# The kernel must be compiled as ELF, so that the bootloader (GRUB or
# SYSLINUX) can recognize it.
#
#-----------------------------------------------------------------------
.section .text
#-----------------------------------------------------------------------
#
# At the beginning there is the code, but inside the code there is also
# the multiboot header. This is why we start with a jump.
#
kstartup:
    jmp start
#
# Here is the multiboot header, that must be placed near the beginning
# of the image-file, but aligned at a multiple of four bytes.
#
.align 4
multiboot_header:
    .int 0x1BADB002                     # magic
    .int 0x00000007                     # flags
    .int -(0x1BADB002 + 0x00000007)     # checksum
#
# Here starts really the code.
#
start:
    #
    # Set ESP at the stack bottom.
    #
    movl $_k_stack_bottom, %esp
    #
    # Reset flags inside EFLAGS, but must use the stack to make it.
    #
    pushl $0
    popf
    #
    # Call function `kmain()'. It is not the usual `main()' because
    # we need to pass some data, but it is not compatible with the
    # standard `main()' prototype.
    #
    # void kmain (uint32_t magic, multiboot_t *info);
    #
    pushl %ebx          # Pointer to the structure containing
                        # data from the boot system.
    pushl %eax          # Boot system signature.
    #
    call kmain          # Do the call.
#
# Halt procedure.
#
halt:
    hlt                 # If the function `kmain()' return, this
                        # instruction will halt the CPU.
    jmp halt            # if the CPU will resume working, the HLT
                        # instruction will be repeated.
#-----------------------------------------------------------------------
.align 4
.section .data
#-----------------------------------------------------------------------
.align 4
.section .bss
#-----------------------------------------------------------------------
