#include <kernel/memory.h>
#include <kernel/ibm_i386.h>
#include <sys/os32.h>
#include <kernel/lib_k.h>
//----------------------------------------------------------------------
#define DEBUG 0
//----------------------------------------------------------------------
static int mb_block_set0 (int block);
//----------------------------------------------------------------------
void
mb_free (addr_t address, size_t size)
{
    unsigned int bstart;
    unsigned int bsize;
    unsigned int bend;
    unsigned int i;
    addr_t       block_address;
    //
    //  k_printf ("releasing 0x%x, size 0x%x\n", (int) address,
    //            (int) size);
    //
    if (size == 0)
      {
        //
        // Zero means nothing.
        //
        return;
      }
    //
    // Show what was requested.
    //
    if (DEBUG)
      {
        k_printf ("%s(%i, %zi)", __func__, (unsigned int) address,
                  size);
      }
    //
    // Calculate size in term of blocks.
    //
    if (size % MEM_BLOCK_SIZE)
      {
        bsize  = size / MEM_BLOCK_SIZE + 1;
      }
    else
      {
        bsize  = size / MEM_BLOCK_SIZE;
      }
    //
    // Calculate start address in term of blocks.
    //
    bstart = address / MEM_BLOCK_SIZE;
    //
    // Calculate end address, in term of blocks.
    // This address is after the memory area to
    // be released.
    //
    bend   = bstart + bsize;
    //
    // Scan the memory map to free memory blocks.
    //
    for (i = bstart; i < bend; i++)
      {
        if (mb_block_set0 (i))
          {
            ;
          }
        else
          {
            block_address = i;
            block_address *= MEM_BLOCK_SIZE;
            k_printf ("[%s] Kernel alert: mem block 0x%x, at address "
                      "0x%x, already released!\n",
                      __FILE__, i, (unsigned int) block_address);
          }
      }
}
//----------------------------------------------------------------------
static int
mb_block_set0 (int block)
{
    int i         = block / 32;
    int j         = block % 32;
    uint32_t mask = 0x80000000 >> j;
    if (mb_table[i] & mask)
      {
        mb_table[i] = mb_table[i] & ~mask;
        return (1);
      }
    else
      {
        return (0);     // The block is already set to 0 inside the map!
      }
}
