#ifndef _KERNEL_NET_ARP_H
#define _KERNEL_NET_ARP_H    1
//----------------------------------------------------------------------
#include <stdint.h>
#include <sys/types.h>
#include <kernel/net/ip.h>
#include <arpa/inet.h>
//----------------------------------------------------------------------
#define ARP_HW_ETHERNET    1
#define ARP_HW_IEEE802     6            // Example, but not used.
//----------------------------------------------------------------------
#define ARP_TYPE_REQUEST   1
#define ARP_TYPE_REPLY     2
//----------------------------------------------------------------------
typedef struct {
  uint16_t hardware_type;
  uint16_t protocol_type;
  uint8_t  hardware_address_length;
  uint8_t  protocol_address_length;
  uint16_t opcode;
  uint8_t  sender_mac[NET_ETHERNET_ADDRESS_LENGTH];
  uint32_t sender_ip;  // Network byte order.
  uint8_t  target_mac[NET_ETHERNET_ADDRESS_LENGTH];
  uint32_t target_ip;  // Network byte order.
  uint8_t  filler[18];                          // [1]
} __attribute__ ((packed)) arp_packet_t;
//
// [1] The filler is just big enough to get a minimal Ethernet frame
//     size.
//
//----------------------------------------------------------------------
#define ARP_MAX_ITEMS 64
#define ARP_MAX_TIME  300       // Seconds.
//
typedef struct {
  time_t    time;
  uint8_t   mac[NET_ETHERNET_ADDRESS_LENGTH];
  h_addr_t  ip;         // Host byte order.
} arp_t;
//
extern arp_t arp_table[ARP_MAX_ITEMS];
//----------------------------------------------------------------------
void   arp_clean     (void);
int    arp_index     (unsigned char mac[6], h_addr_t ip);
void   arp_init      (void);
void   arp_print     (void);
arp_t *arp_reference (void);
void   arp_request   (h_addr_t ip);
int    arp_rx        (int n, int f);
//----------------------------------------------------------------------
#endif
