#include <kernel/net.h>
#include <kernel/net/arp.h>
#include <kernel/net/ip.h>
#include <sys/os32.h>
#include <kernel/lib_k.h>
#include <errno.h>
#include <arpa/inet.h>
//----------------------------------------------------------------------
#define DEBUG 0
//----------------------------------------------------------------------
int
arp_rx (int n, int f)
{
  net_ethernet_frame_t *frame = &net_table[n].ethernet.buffer[f].frame;
  arp_packet_t         *arp   = (arp_packet_t *) &frame->packet;
  int                   i;
  int                   a;          // ARP table index.
  arp_t                *arp_table_new_item;
  //
  net_ethernet_frame_t  ans_frame;
  arp_packet_t         *ans_arp = (arp_packet_t *) &ans_frame.packet;
  //
  //
  //
  if (n >= NET_MAX_DEVICES || n < 0)
    {
      errset (EINVAL);          // Invalid argument.
      return (-1);
    }
  //
  if (!(net_table[n].type & NET_DEV_ETH))
    {
      errset (EINVAL);          // Invalid argument.
      return (-1);
    }
  //
  if (ntohs (frame->header.type) != NET_PROT_ARP)
    {
      errset (EINVAL);          // Invalid argument.
      return (-1);
    }
  //
  //
  //
  if (ntohs (arp->opcode) == ARP_TYPE_REQUEST)
    {
      //
      // This is an ARP request: we try to answare if the
      // the IP address is owned.
      //
      if (arp->target_ip == htonl (net_table[n].ip))
        {
          //
          // Found IPv4 address. Prepare an answare.
          //
          memcpy (ans_frame.header.dst, arp->sender_mac,
                  (size_t) NET_ETHERNET_ADDRESS_LENGTH);
          memcpy (ans_frame.header.src, net_table[n].ethernet.mac,
                  (size_t) NET_ETHERNET_ADDRESS_LENGTH);
          ans_frame.header.type  = htons (NET_PROT_ARP);
          ans_arp->hardware_type = htons (ARP_HW_ETHERNET);
          ans_arp->protocol_type = htons (NET_PROT_IP);
          ans_arp->hardware_address_length
                                 = NET_ETHERNET_ADDRESS_LENGTH;
          ans_arp->protocol_address_length
                                 = NET_IP_ADDRESS_LENGTH;
          ans_arp->opcode        = htons (ARP_TYPE_REPLY);
          memcpy (ans_arp->sender_mac, net_table[n].ethernet.mac,
                  (size_t) NET_ETHERNET_ADDRESS_LENGTH);
          ans_arp->sender_ip     = htonl (net_table[n].ip);
          memcpy (ans_arp->target_mac, arp->sender_mac,
                  (size_t) NET_ETHERNET_ADDRESS_LENGTH);
          ans_arp->target_ip     = arp->sender_ip;
          for (i = 0; i < (sizeof_array (ans_arp->filler)); i++)
            {
              ans_arp->filler[i] = 0;
            }
          //
          // Send it.
          //
          net_eth_tx (n, &ans_frame, sizeof (arp_packet_t) + 14);
        }
      //
      // Done.
      //
      return (0);
    }
  //
  //
  //
  if (ntohs (arp->opcode) == ARP_TYPE_REPLY)
    {
      //
      // We should save it inside the ARP table.
      //
      for (a = 0; a < ARP_MAX_ITEMS; a++)
        {
          //
          // Check if we already have the same item.
          //
          if (memcmp (arp->sender_mac, arp_table[a].mac,
                      (size_t) NET_ETHERNET_ADDRESS_LENGTH) == 0)
            {
              if (arp_table[a].ip == ntohl (arp->sender_ip))
                {
                  //
                  // Found: update the time.
                  //
                  arp_table[a].time = k_time (NULL);
                  //
                  // Done.
                  //
                  return (0);
                }
              else
                {
                  //
                  // There is already the Ethernet address,
                  // but the IP is different.
                  //
                  if (DEBUG)
                    {
                      k_printf ("%s:%i: Ethernet address "
                                "%x02:%02:%x02:%x02:%02:%x02 "
                                "has more than one IP\n",
                                __FILE__, __LINE__,
                                arp_table[a].mac[0],
                                arp_table[a].mac[1],
                                arp_table[a].mac[2],
                                arp_table[a].mac[3],
                                arp_table[a].mac[4],
                                arp_table[a].mac[5]);
                      //
                      // End of scan.
                      //
                      break;
                    }
                }
            }
        }
        //
        // If we are here, the MAC-IP couple is new: get a new ARP item.
        //
        arp_table_new_item = arp_reference ();
        //
        memcpy (arp_table_new_item->mac, arp->sender_mac,
                (size_t) NET_ETHERNET_ADDRESS_LENGTH);
        arp_table_new_item->ip   = ntohl (arp->sender_ip);
        arp_table_new_item->time = k_time (NULL);
        //
        // Done.
        //
        return (0);
    }
  //
  // If we are here, we don't know the ARP type.
  //
  if (DEBUG)
    {
      k_printf ("%s:%i: unknown ARP type: %i\n",
                (int) ntohs (arp->opcode));
    }
  //
  // Done.
  //
  return (0);
}
