#include <arpa/inet.h>
#include <kernel/net.h>
#include <kernel/net/arp.h>
#include <sys/os32.h>
#include <kernel/lib_k.h>
//----------------------------------------------------------------------
#define DEBUG 0
//----------------------------------------------------------------------
int
net_rx (void)
{
  int n;                        // NET table index.
  int b;                        // Frame index.
  net_ethernet_frame_t *frame;
  int counter = 0;
  //
  // Scan NET table.
  //
  for (n = 0; n < NET_MAX_DEVICES; n++)
    {
      //
      // Ethernet.
      //
      if (net_table[n].type & NET_DEV_ETH)
        {
          for (b = 0; b < NET_MAX_BUFFERS; b++)
            {
              if (net_table[n].ethernet.buffer[b].clock > 0)
                {
                  frame = (net_ethernet_frame_t *)
                    & net_table[n].ethernet.buffer[b].frame;
                  //
                  if (ntohs (frame->header.type) == NET_PROT_ARP)
                    {
                      arp_rx (n, b);
                      //
                      // Remove packet from buffer.
                      //
                      net_table[n].ethernet.buffer[b].clock = 0;
                      //
                      // Increment the packet received counter.
                      //
                      counter++;
                    }
                  else if (ntohs (frame->header.type) == NET_PROT_IP)
                    {
                      ip_rx (n, b);
                      //
                      // Remove packet from buffer.
                      //
                      net_table[n].ethernet.buffer[b].clock = 0;
                      //
                      // Increment the packet received counter.
                      //
                      counter++;
                    }
                  else
                    {
                      //
                      // Unknown frame type.
                      //
                      k_printf ("received an unknown frame type %04x\n",
                                ntohs (frame->header.type));
                      //
                      // Remove packet from buffer.
                      //
                      net_table[n].ethernet.buffer[b].clock = 0;
                      //
                      // Increment the packet received counter anyway.
                      //
                      counter++;
                    }
                }
            }
        }
      //
      // Loopback
      //
      else if (net_table[n].type & NET_DEV_LOOP)
        {
          for (b = 0; b < NET_MAX_BUFFERS; b++)
            {
              if (net_table[n].loopback.buffer[b].clock > 0)
                {
                  ip_rx (n, b);
                  //
                  // Remove packet from buffer.
                  //
                  net_table[n].loopback.buffer[b].clock = 0;
                  //
                  // Increment the packet received counter.
                  //
                  counter++;
                  //
                }
            }
        }
    }
  //
  // Remove ARP items that are too old.
  //
  arp_clean ();
  //
  //
  //
  return (counter);
}
