#include <kernel/net.h>
#include <kernel/net/ip.h>
#include <kernel/net/route.h>
#include <kernel/net/tcp.h>
#include <sys/os32.h>
#include <kernel/lib_k.h>
#include <kernel/fs.h>
#include <errno.h>
#include <stdlib.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
//----------------------------------------------------------------------
int
tcp_status (void *ip_packet)
{
  struct iphdr   *iphdr;
  struct tcphdr  *tcphdr;
  int    s;
  //
  if (ip_packet == NULL)
    {
      errset (EINVAL);
      return (-1);
    }
  //
  iphdr  = ip_packet;
  tcphdr = (struct tcphdr *)
           &(((uint8_t *) ip_packet)[iphdr->ihl * 4]);
  //
  if (iphdr->saddr == 0 || iphdr->daddr == 0)
    {
      errset (EINVAL);
      return (-1);
    }
  //
  if (tcphdr->source == 0 || tcphdr->dest == 0)
    {
      errset (EINVAL);
      return (-1);
    }
  //
  // Find a connection with the same IPs and ports.
  //
  for (s = 0; s < SOCK_MAX_SLOTS; s++)
    {
      if (!sock_table[s].active)                 continue;
      if (sock_table[s].family != AF_INET)       continue;
      if (sock_table[s].protocol != IPPROTO_TCP) continue;
      if (sock_table[s].unreach_port)            continue;
      if (sock_table[s].unreach_host)            continue;
      if (sock_table[s].laddr != ntohl (iphdr->daddr))   continue;
      if (sock_table[s].lport != ntohs (tcphdr->dest))   continue;
      if (sock_table[s].raddr != ntohl (iphdr->saddr))   continue;
      if (sock_table[s].rport != ntohs (tcphdr->source)) continue;
      //
      // A corresponding socket was found.
      //
      return ((int) sock_table[s].tcp.conn);
    }
  //
  // Socket not found.
  //
  return (0);
}
