#include <kernel/proc.h>
#include <kernel/lib_k.h>
#include <kernel/lib_s.h>
#include <kernel/driver/nic/ne2k.h>
#include <kernel/net.h>
#include <kernel/driver/pci.h>
//----------------------------------------------------------------------
#define DEBUG 0
//----------------------------------------------------------------------
void
proc_sch_net (void)
{
  int n;                        // NET table index.
  int counter = 0;
  int pid;
  int tcp_wake_up;
  //
  // Do what there is to do with network interfaces, in particular get
  // received frames, into the `net_table[n]...buffer[f]' table.
  //
  for (n = 0; n < NET_MAX_DEVICES; n++)
    {
      if (net_table[n].type == NET_DEV_ETH_NE2K)
        {
          //
          // The function `ne2k_isr()' will call also `ne2k_rx()'
          // that is responsible for placing Ethernet frames inside
          // `ethernet_table[eth].frame_info[f]'.
          //
          ne2k_isr (net_table[n].ethernet.base_io);
        }
      else if (net_table[n].type == NET_DEV_LOOPBACK)
        {
          //
          // Packets should be already inside the packet buffer table.
          //
          ;
        }
    }
  //
  // Do something with received Ethernet frames.
  //
  counter = net_rx ();
  //
  if (DEBUG)
    {
      if (counter)
        {
          k_printf ("+ %i packet(s) received\n", counter);
        }
    }
  //
  // Do something with TCP connections.
  //
  tcp_wake_up = tcp ();
  //
  // Wake up sleeping processes.
  //
  if (counter || tcp_wake_up == TCP_TRY_READ)
    {
      //
      // Wake up sleeping processes, waiting to read from a socket.
      //
      if (DEBUG)
        {
          k_printf ("wake up processes, waiting for a socket!\n");
        }
      for (pid = 1; pid < PROCESS_MAX; pid++)
        {
          if (proc_table[pid].status == PROC_SLEEPING
              && (proc_table[pid].wakeup_events
                  & WAKEUP_EVENT_SOCK_READ))
            {
              proc_table[pid].wakeup_events = 0;
              proc_table[pid].status = PROC_READY;
            }
        }
    }
  //
  if (tcp_wake_up == TCP_TRY_WRITE)
    {
      //
      // Wake up sleeping processes, waiting to write to a socket.
      //
      for (pid = 1; pid < PROCESS_MAX; pid++)
        {
          if (proc_table[pid].status == PROC_SLEEPING
              && (proc_table[pid].wakeup_events
                  & WAKEUP_EVENT_SOCK_WRITE))
            {
              proc_table[pid].wakeup_events = 0;
              proc_table[pid].status = PROC_READY;
            }
        }
    }
}
