#ifndef _ERRNO_H
#define _ERRNO_H       1
//----------------------------------------------------------------------
#include <limits.h>
#include <string.h>
#include <sys/os32.h>
#include <kernel/lib_k.h>

//----------------------------------------------------------------------
// The variable `errno' is standard, but `errln' and `errfn' are added
// to keep track of the error source. Variable `errln' is used to save
// the source file line number; variable `errfn' is used to save the
// source file name. To set these variable in a consistent way it is
// also added a macro-instruction: `errset'.
//----------------------------------------------------------------------
extern int   errno;
extern int   errln;
extern char  errfn[PATH_MAX];
//
#define errset(e)       (errln = __LINE__, \
                         strncpy (errfn, __FILE__, PATH_MAX), \
                         errno = e)
//
//#define errset(e)       (errln = __LINE__,
//                         strncpy (errfn, __FILE__, PATH_MAX),
//                         errno = e,
//                         k_perror (NULL))
//
//----------------------------------------------------------------------
// Standard POSIX `errno' macro variables.
//----------------------------------------------------------------------
#define E2BIG            1 // Argument list too long.
#define EACCES           2 // Permission denied.
#define EADDRINUSE       3 // Address in use.
#define EADDRNOTAVAIL    4 // Address not available.
#define EAFNOSUPPORT     5 // Address family not supported.
#define EAGAIN           6 // Resource unavailable, try again.
#define EALREADY         7 // Connection already in progress.
#define EBADF            8 // Bad file descriptor.
#define EBADMSG          9 // Bad message.
#define EBUSY           10 // Device or resource busy.
#define ECANCELED       11 // Operation canceled.
#define ECHILD          12 // No child processes.
#define ECONNABORTED    13 // Connection aborted.
#define ECONNREFUSED    14 // Connection refused.
#define ECONNRESET      15 // Connection reset.
#define EDEADLK         16 // Resource deadlock would occur.
#define EDESTADDRREQ    17 // Destination address required.
#define EDOM            18 // Mathematics argument out of domain of
                           // function.
#define EDQUOT          19 // Reserved.
#define EEXIST          20 // File exists.
#define EFAULT          21 // Bad address.
#define EFBIG           22 // File too large.
#define EHOSTUNREACH    23 // Host is unreachable.
#define EIDRM           24 // Identifier removed.
#define EILSEQ          25 // Illegal byte sequence.
#define EINPROGRESS     26 // Operation in progress.
#define EINTR           27 // Interrupted function.
#define EINVAL          28 // Invalid argument.
#define EIO             29 // I/O error.
#define EISCONN         30 // Socket is connected.
#define EISDIR          31 // Is a directory.
#define ELOOP           32 // Too many levels of symbolic links.
#define EMFILE          33 // Too many open files.
#define EMLINK          34 // Too many links.
#define EMSGSIZE        35 // Message too large.
#define EMULTIHOP       36 // Reserved.
#define ENAMETOOLONG    37 // Filename too long.
#define ENETDOWN        38 // Network is down.
#define ENETRESET       39 // Connection aborted by network.
#define ENETUNREACH     40 // Network unreachable.
#define ENFILE          41 // Too many files open in system.
#define ENOBUFS         42 // No buffer space available.
#define ENODATA         43 // No message is available on the stream head
                           // read queue.
#define ENODEV          44 // No such device.
#define ENOENT          45 // No such file or directory.
#define ENOEXEC         46 // Executable file format error.
#define ENOLCK          47 // No locks available.
#define ENOLINK         48 // Reserved.
#define ENOMEM          49 // Not enough space.
#define ENOMSG          50 // No message of the desired type.
#define ENOPROTOOPT     51 // Protocol not available.
#define ENOSPC          52 // No space left on device.
#define ENOSR           53 // No stream resources.
#define ENOSTR          54 // Not a stream.
#define ENOSYS          55 // Function not supported.
#define ENOTCONN        56 // The socket is not connected.
#define ENOTDIR         57 // Not a directory.
#define ENOTEMPTY       58 // Directory not empty.
#define ENOTSOCK        59 // Not a socket.
#define ENOTSUP         60 // Not supported.
#define ENOTTY          61 // Inappropriate I/O control operation.
#define ENXIO           62 // No such device or address.
#define EOPNOTSUPP      63 // Operation not supported on socket.
#define EOVERFLOW       64 // Value too large to be stored in data type.
#define EPERM           65 // Operation not permitted.
#define EPIPE           66 // Broken pipe.
#define EPROTO          67 // Protocol error.
#define EPROTONOSUPPORT 68 // Protocol not supported.
#define EPROTOTYPE      69 // Protocol wrong type for socket.
#define ERANGE          70 // Result too large.
#define EROFS           71 // Read-only file system.
#define ESPIPE          72 // Invalid seek.
#define ESRCH           73 // No such process.
#define ESTALE          74 // Reserved.
#define ETIME           75 // Stream ioctl() timeout.
#define ETIMEDOUT       76 // Connection timed out.
#define ETXTBSY         77 // Text file busy.
#define EWOULDBLOCK     78 // Operation would block
                           // (may be the same as EAGAIN).
#define EXDEV           79 // Cross-device link.
//----------------------------------------------------------------------
// Added os32 errors.
//----------------------------------------------------------------------
#define EUNKNOWN                   (-1) // Unknown error.
#define E_NO_MEDIUM                 80  // No medium found.
#define E_MEDIUM                    81  // Medium reported error.
#define E_FILE_TYPE                 82  // File type not compatible.
#define E_ROOT_INODE_NOT_CACHED     83  // The root directory inode is
                                        // not cached.
#define E_CANNOT_READ_SUPERBLOCK    84  // Cannot read super block.
#define E_MAP_INODE_TOO_BIG         85  // Map inode too big.
#define E_MAP_ZONE_TOO_BIG          86  // Map zone too big.
#define E_DATA_ZONE_TOO_BIG         87  // Data zone too big.
#define E_CANNOT_FIND_ROOT_DEVICE   88  // Cannot find root device.
#define E_CANNOT_FIND_ROOT_INODE    89  // Cannot find root inode.
#define E_FILE_TYPE_UNSUPPORTED     90  // File type unsupported.
#define E_ENV_TOO_BIG               91  // Environment too big.
#define E_LIMIT                     92  // Exceeded implementation
                                        // limits.
#define E_NOT_MOUNTED               93  // Not mounted.
#define E_NOT_IMPLEMENTED           94  // Not implemented.
#define E_HARDWARE_FAULT            95  // Hardware fault.
#define E_DRIVER_FAULT              96  // Driver fault.
#define E_PIPE_FULL                 97  // Pipe full.
#define E_PIPE_EMPTY                98  // Pipe empty.
#define E_PART_TYPE_NOT_MINIX       99  // Not a Minix partition type.
#define E_FS_TYPE_NOT_SUPPORTED    100  // File system type not
                                        // supported.
#define E_PDU_TOO_BIG              101  // PDU too big.
#define E_ARP_MISSING              102  // ARP missing address.
//----------------------------------------------------------------------
// Default descriptions for errors.
//----------------------------------------------------------------------
#define TEXT_E2BIG                      "Argument list too long."
#define TEXT_EACCES                     "Permission denied."
#define TEXT_EADDRINUSE                 "Address in use."
#define TEXT_EADDRNOTAVAIL              "Address not available."
#define TEXT_EAFNOSUPPORT               "Address family not supported."
#define TEXT_EAGAIN                     "Resource unavailable, " \
                                        "try again."
#define TEXT_EALREADY                   "Connection already in " \
                                        "progress."
#define TEXT_EBADF                      "Bad file descriptor."
#define TEXT_EBADMSG                    "Bad message."
#define TEXT_EBUSY                      "Device or resource busy."
#define TEXT_ECANCELED                  "Operation canceled."
#define TEXT_ECHILD                     "No child processes."
#define TEXT_ECONNABORTED               "Connection aborted."
#define TEXT_ECONNREFUSED               "Connection refused."
#define TEXT_ECONNRESET                 "Connection reset."
#define TEXT_EDEADLK                    "Resource deadlock would occur."
#define TEXT_EDESTADDRREQ               "Destination address required."
#define TEXT_EDOM                       "Mathematics argument out of " \
                                        "domain of function."
#define TEXT_EDQUOT                     "Reserved error: EDQUOT"
#define TEXT_EEXIST                     "File exists."
#define TEXT_EFAULT                     "Bad address."
#define TEXT_EFBIG                      "File too large."
#define TEXT_EHOSTUNREACH               "Host is unreachable."
#define TEXT_EIDRM                      "Identifier removed."
#define TEXT_EILSEQ                     "Illegal byte sequence."
#define TEXT_EINPROGRESS                "Operation in progress."
#define TEXT_EINTR                      "Interrupted function."
#define TEXT_EINVAL                     "Invalid argument."
#define TEXT_EIO                        "I/O error."
#define TEXT_EISCONN                    "Socket is connected."
#define TEXT_EISDIR                     "Is a directory."
#define TEXT_ELOOP                      "Too many levels of " \
                                        "symbolic links."
#define TEXT_EMFILE                     "Too many open files."
#define TEXT_EMLINK                     "Too many links."
#define TEXT_EMSGSIZE                   "Message too large."
#define TEXT_EMULTIHOP                  "Reserved error: EMULTIHOP"
#define TEXT_ENAMETOOLONG               "Filename too long."
#define TEXT_ENETDOWN                   "Network is down."
#define TEXT_ENETRESET                  "Connection aborted by network."
#define TEXT_ENETUNREACH                "Network unreachable."
#define TEXT_ENFILE                     "Too many files open in system."
#define TEXT_ENOBUFS                    "No buffer space available."
#define TEXT_ENODATA                    "No message is available on " \
                                        "the stream head read queue."
#define TEXT_ENODEV                     "No such device."
#define TEXT_ENOENT                     "No such file or directory."
#define TEXT_ENOEXEC                    "Executable file format error."
#define TEXT_ENOLCK                     "No locks available."
#define TEXT_ENOLINK                    "Reserved error: ENOLINK"
#define TEXT_ENOMEM                     "Not enough space."
#define TEXT_ENOMSG                     "No message of the desired " \
                                        "type."
#define TEXT_ENOPROTOOPT                "Protocol not available."
#define TEXT_ENOSPC                     "No space left on device."
#define TEXT_ENOSR                      "No stream resources."
#define TEXT_ENOSTR                     "Not a stream."
#define TEXT_ENOSYS                     "Function not supported."
#define TEXT_ENOTCONN                   "The socket is not connected."
#define TEXT_ENOTDIR                    "Not a directory."
#define TEXT_ENOTEMPTY                  "Directory not empty."
#define TEXT_ENOTSOCK                   "Not a socket."
#define TEXT_ENOTSUP                    "Not supported."
#define TEXT_ENOTTY                     "Inappropriate I/O control " \
                                        "operation."
#define TEXT_ENXIO                      "No such device or address."
#define TEXT_EOPNOTSUPP                 "Operation not supported on " \
                                        "socket."
#define TEXT_EOVERFLOW                  "Value too large to be " \
                                        "stored in data type."
#define TEXT_EPERM                      "Operation not permitted."
#define TEXT_EPIPE                      "Broken pipe."
#define TEXT_EPROTO                     "Protocol error."
#define TEXT_EPROTONOSUPPORT            "Protocol not supported."
#define TEXT_EPROTOTYPE                 "Protocol wrong type for " \
                                        "socket."
#define TEXT_ERANGE                     "Result too large."
#define TEXT_EROFS                      "Read-only file system."
#define TEXT_ESPIPE                     "Invalid seek."
#define TEXT_ESRCH                      "No such process."
#define TEXT_ESTALE                     "Reserved error: ESTALE"
#define TEXT_ETIME                      "Stream ioctl() timeout."
#define TEXT_ETIMEDOUT                  "Connection timed out."
#define TEXT_ETXTBSY                    "Text file busy."
#define TEXT_EWOULDBLOCK                "Operation would block."
#define TEXT_EXDEV                      "Cross-device link."
//----------------------------------------------------------------------
#define TEXT_EUNKNOWN                   "Unknown error."
#define TEXT_E_NO_MEDIUM                "No medium found."
#define TEXT_E_MEDIUM                   "Medium reported error"
#define TEXT_E_FILE_TYPE                "File type not compatible."
#define TEXT_E_ROOT_INODE_NOT_CACHED    "The root directory inode " \
                                        "is not cached."
#define TEXT_E_CANNOT_READ_SUPERBLOCK   "Cannot read super block."
#define TEXT_E_MAP_INODE_TOO_BIG        "Map inode too big."
#define TEXT_E_MAP_ZONE_TOO_BIG         "Map zone too big."
#define TEXT_E_DATA_ZONE_TOO_BIG        "Data zone too big."
#define TEXT_E_CANNOT_FIND_ROOT_DEVICE  "Cannot find root device."
#define TEXT_E_CANNOT_FIND_ROOT_INODE   "Cannot find root inode."
#define TEXT_E_FILE_TYPE_UNSUPPORTED    "File type unsupported."
#define TEXT_E_ENV_TOO_BIG              "Environment too big."
#define TEXT_E_LIMIT                    "Exceeded implementation " \
                                        "limits."
#define TEXT_E_NOT_MOUNTED              "Not mounted."
#define TEXT_E_NOT_IMPLEMENTED          "Not implemented."
#define TEXT_E_HARDWARE_FAULT           "Hardware fault."
#define TEXT_E_DRIVER_FAULT             "Driver fault."
#define TEXT_E_PIPE_FULL                "Pipe full."
#define TEXT_E_PIPE_EMPTY               "Pipe empty."
#define TEXT_E_PART_TYPE_NOT_MINIX      "Not a Minix partition type."
#define TEXT_E_FS_TYPE_NOT_SUPPORTED    "File system type not " \
                                        "supported."
#define TEXT_E_PDU_TOO_BIG              "PDU too big."
#define TEXT_E_ARP_MISSING              "ARP missing address."
//----------------------------------------------------------------------
#endif
