#ifndef _STDINT_H
#define _STDINT_H       1
//----------------------------------------------------------------------
typedef signed char             int8_t;
typedef short int               int16_t;
typedef int                     int32_t;
typedef long long int           int64_t;
//
typedef unsigned char           uint8_t;
typedef unsigned short int      uint16_t;
typedef unsigned int            uint32_t;
typedef unsigned long long int  uint64_t;
//
#define INT8_MIN              (-0x80)
#define INT16_MIN             (-0x8000)
#define INT32_MIN             (-0x80000000)
#define INT64_MIN             (-0x8000000000000000LL)
//
#define INT8_MAX                0x7F
#define INT16_MAX               0x7FFF
#define INT32_MAX               0x7FFFFFFF
#define INT64_MAX               0x7FFFFFFFFFFFFFFFLL
//
#define UINT8_MAX               0xFF
#define UINT16_MAX              0xFFFF
#define UINT32_MAX              0xFFFFFFFFU
#define UINT64_MAX              0xFFFFFFFFFFFFFFFFULL
//----------------------------------------------------------------------
typedef signed char             int_least8_t;
typedef short int               int_least16_t;
typedef int                     int_least32_t;
typedef long long int           int_least64_t;
//
typedef unsigned char           uint_least8_t;
typedef unsigned short int      uint_least16_t;
typedef unsigned int            uint_least32_t; // x86-32
typedef unsigned long long int  uint_least64_t; // x86-32
//
#define INT_LEAST8_MIN        (-0x80)
#define INT_LEAST16_MIN       (-0x8000)
#define INT_LEAST32_MIN       (-0x80000000)
#define INT_LEAST64_MIN       (-0x8000000000000000LL)
//
#define INT_LEAST8_MAX          0x7F
#define INT_LEAST16_MAX         0x7FFF
#define INT_LEAST32_MAX         0x7FFFFFFF
#define INT_LEAST64_MAX         0x7FFFFFFFFFFFFFFFLL
//
#define UINT_LEAST8_MAX         0xFF
#define UINT_LEAST16_MAX        0xFFFF
#define UINT_LEAST32_MAX        0xFFFFFFFFU
#define UINT_LEAST64_MAX        0xFFFFFFFFFFFFFFFFULL
//----------------------------------------------------------------------
#define INT8_C(VAL)             VAL
#define INT16_C(VAL)            VAL
#define INT32_C(VAL)            VAL
#define INT64_C(VAL)            VAL ## LL
//
#define UINT8_C(VAL)            VAL
#define UINT16_C(VAL)           VAL
#define UINT32_C(VAL)           VAL ## U
#define UINT64_C(VAL)           VAL ## ULL
//----------------------------------------------------------------------
typedef signed char             int_fast8_t;
typedef int                     int_fast16_t;
typedef int                     int_fast32_t;
typedef long long int           int_fast64_t;
//
typedef unsigned char           uint_fast8_t;
typedef unsigned int            uint_fast16_t;
typedef unsigned int            uint_fast32_t;
typedef unsigned long long int  uint_fast64_t;
//
#define INT_FAST8_MIN         (-0x80)
#define INT_FAST16_MIN        (-0x80000000)
#define INT_FAST32_MIN        (-0x80000000)
#define INT_FAST64_MIN        (-0x8000000000000000LL)
//
#define INT_FAST8_MAX           0x7F
#define INT_FAST16_MAX          0x7FFFFFFF
#define INT_FAST32_MAX          0x7FFFFFFF
#define INT_FAST64_MAX          0x7FFFFFFFFFFFFFFFLL
//
#define UINT_FAST8_MAX          0xFF
#define UINT_FAST16_MAX         0xFFFFFFFFU
#define UINT_FAST32_MAX         0xFFFFFFFFU
#define UINT_FAST64_MAX         0xFFFFFFFFFFFFFFFFULL
//----------------------------------------------------------------------
typedef int                     intptr_t;
typedef unsigned int            uintptr_t;
//
#define INTPTR_MIN            (-0x80000000)
#define INTPTR_MAX              0x7FFFFFFF
#define UINTPTR_MAX             0xFFFFFFFFU
//
typedef long long int           intmax_t;
typedef unsigned long long int  uintmax_t;
//
#define INTMAX_C(VAL)           VAL ## LL
#define UINTMAX_C(VAL)          VAL ## ULL
#define INTMAX_MIN   (-INTMAX_C(0x8000000000000000))
#define INTMAX_MAX    (INTMAX_C(0x7FFFFFFFFFFFFFFF))
#define UINTMAX_MAX  (UINTMAX_C(0xFFFFFFFFFFFFFFFF))
//----------------------------------------------------------------------
#define PTRDIFF_MIN           (-0x80000000)
#define PTRDIFF_MAX             0x7FFFFFFF
//
#define SIG_ATOMIC_MIN        (-0x80000000)
#define SIG_ATOMIC_MAX          0x7FFFFFFF
//
#define SIZE_MAX                0xFFFFFFFFU
//
#define WCHAR_MIN               0x00000000
#define WCHAR_MAX               0xFFFFFFFFU
//
#define WINT_MIN              (-0x8000000000000000LL)
#define WINT_MAX                0x7FFFFFFFFFFFFFFFLL
//----------------------------------------------------------------------
#endif
