#include <time.h>
//----------------------------------------------------------------------
static int leap_year (int year);
//----------------------------------------------------------------------
struct tm *
gmtime (const time_t *timer)
{
    static struct tm  tms;
    int               loop;
    unsigned int      remainder;
    unsigned int      days;
    //
    // Check argument.
    //
    if (timer == NULL)
      {
        return (NULL);
      }
    //
    // Days since epoch. There are 86400 seconds per day.
    // At the moment, the field `tm_yday' will contain
    // all days since epoch.
    //
    days      = *timer / 86400L;
    remainder = *timer % 86400L;
    //
    // Minutes, after full days.
    //
    tms.tm_min = remainder / 60U;
    //
    // Seconds, after full minutes.
    //
    tms.tm_sec = remainder % 60U;
    //
    // Hours, after full days.
    //
    tms.tm_hour = tms.tm_min / 60;
    //
    // Minutes, after full hours.
    //
    tms.tm_min = tms.tm_min % 60;
    //
    // Find the week day. Must remove some days to align the
    // calculation. So: the week days of the first week of 1970
    // are not valid! After 1970-01-04 calculations are right.
    //
    tms.tm_wday = (days - 3) % 7;
    //
    // Find the year: the field `tm_yday' will be reduced to the days
    // of current year.
    //
    for (tms.tm_year = 1970; days > 0; tms.tm_year++)
      {
        if (leap_year (tms.tm_year))
          {
            if (days >= 366)
              {
                days -= 366;
                continue;
              }
            else
              {
                break;
              }
          }
        else
          {
            if (days >= 365)
              {
                days -= 365;
                continue;
              }
            else
              {
                break;
              }
          }
      }
    //
    // Day of the year.
    //
    tms.tm_yday = days + 1;
    //
    // Find the month.
    //
    tms.tm_mday = days + 1;
    //
    for (tms.tm_mon = 0, loop = 1; tms.tm_mon <= 12 && loop;)
      {
        tms.tm_mon++;
        //
        switch (tms.tm_mon)
          {
            case 1:
            case 3:
            case 5:
            case 7:
            case 8:
            case 10:
            case 12:
                if (tms.tm_mday >= 31)
                  {
                    tms.tm_mday -= 31;
                  }
                else
                  {
                    loop = 0;
                  }
                break;
            case 4:
            case 6:
            case 9:
            case 11:
                if (tms.tm_mday >= 30)
                  {
                    tms.tm_mday -= 30;
                  }
                else
                  {
                    loop = 0;
                  }
                break;
            case 2:
                if (leap_year (tms.tm_year))
                  {
                    if (tms.tm_mday >= 29)
                      {
                        tms.tm_mday -= 29;
                      }
                    else
                      {
                        loop = 0;
                      }
                  }
                else
                  {
                    if (tms.tm_mday >= 28)
                      {
                        tms.tm_mday -= 28;
                      }
                    else
                      {
                        loop = 0;
                      }
                  }
                break;
          }
      }
    //
    // No check for day light saving time.
    //
    tms.tm_isdst = 0;
    //
    // Return.
    //
    return (&tms);
}
//----------------------------------------------------------------------
static int
leap_year (int year)
{
    if ((year % 4) == 0)
      {
        if ((year % 100) == 0)
          {
            if ((year % 400) == 0)
              {
                return (1);
              }
            else
              {
                return (0);
              }
          }
        else
          {
            return (1);
          }
      }
    else
      {
        return (0);
      }
}
